@@ -46,10 +46,7 @@ ArpScreen::ArpScreen(QWidget* parent) : QWidget (parent)
     minStepWidth = 1.0;
     patternMaxIndex = 0;
     isMuted = false;
-
-    timer = new QTimer(this);
-    timer->setSingleShot(true);
-    connect(timer, SIGNAL(timeout()), this, SLOT(update()));
+    needsRedraw = false;
 }
 
 ArpScreen::~ArpScreen()
@@ -84,6 +81,7 @@ void ArpScreen::paintEvent(QPaintEvent*)
     int patternLen = pattern.length();
     bool chordmd = false;
     int octave = 0;
+    int semitone = 0;
     double vel =1.0;
     double v = 0;
     int grv_cur_sft = 0;
@@ -91,6 +89,7 @@ void ArpScreen::paintEvent(QPaintEvent*)
     int grv_cur_vel = 0;
     int grooveIndex = 0;
     int chordindex = 0;
+    int polyindex = 0;
     l2 = 0;
     QChar c;
 
@@ -104,7 +103,6 @@ void ArpScreen::paintEvent(QPaintEvent*)
     p.setViewport(0, 0, w, h);
     p.setWindow(0, 0, w, h);
     p.setPen(QColor(20, 160, 20));
-    p.drawRect(0, 0, w - 1, h - 1);
 
     //Grid
     len = nSteps;
@@ -163,6 +161,7 @@ void ArpScreen::paintEvent(QPaintEvent*)
     stepwd = 1.0;
     vel = 0.8;
     octave = 0;
+    semitone = 0;
     chordmd = false;
     chordindex = 0;
 
@@ -173,6 +172,7 @@ void ArpScreen::paintEvent(QPaintEvent*)
         if (c.isDigit())
         {
             nlines = c.digitValue() + 1;
+            if (chordmd && (nlines == 1) && chordindex) polyindex++;
             if (!chordindex)
             {
                 if (chordmd) chordindex++;
@@ -220,8 +220,17 @@ void ArpScreen::paintEvent(QPaintEvent*)
                     octave--;
                     break;
 
+                case 't':
+                    semitone++;
+                    break;
+
+                case 'g':
+                    semitone--;
+                    break;
+
                 case '=':
                     octave=0;
+                    semitone=0;
                     break;
 
                 case '/':
@@ -257,11 +266,15 @@ void ArpScreen::paintEvent(QPaintEvent*)
 
             if (nlines > 0) {
                 pen.setWidth(notestreak_thick);
-                pen.setColor(QColor(80 + 60 * v, 160 + 40 * v, 80 + 60 * v));
+                if (semitone != 0)
+                    pen.setColor(QColor(50 + 60 * v, 130 + 40 * v, abs(100 + 10 * semitone) % 256));
+                else
+                    pen.setColor(QColor(80 + 60 * v, 160 + 40 * v, 80 + 60 * v));
                 p.setPen(pen);
                 ypos = yscale - yscale * (nlines - 1 + octYoffset)
                             / (patternMaxIndex + 1) / noctaves
-                            + ARPSCR_VMARG - 3 + notestreak_thick;
+                            + ARPSCR_VMARG - 3 + notestreak_thick
+                            - notestreak_thick * polyindex;
                 xpos = ARPSCR_HMARG + x + pen.width() / 2;
                 p.drawLine(xpos, ypos, xpos + dx - pen.width(), ypos);
                 // Cursor
@@ -287,13 +300,19 @@ void ArpScreen::updateScreen(const QString& p_pattern, int p_minOctave,
     minOctave = p_minOctave;
     nSteps = p_nSteps;
     patternMaxIndex = p_patternMaxIndex;
-
-    update();
+    needsRedraw = true;
 }
 
 void ArpScreen::updateScreen(int p_index)
 {
     currentIndex = p_index;
+    needsRedraw = true;
+}
+
+void ArpScreen::updateDraw()
+{
+    if (!needsRedraw) return;
+    needsRedraw = false;
     update();
 }
 
@@ -302,13 +321,13 @@ void ArpScreen::newGrooveValues(int tick, int vel, int length)
     grooveTick = tick;
     grooveVelocity = vel;
     grooveLength = length;
-    if (!timer->isActive()) timer->start(50);
+    needsRedraw = true;
 }
 
 void ArpScreen::setMuted(bool on)
 {
     isMuted = on;
-    update();
+    needsRedraw = true;
 }
 
 QSize ArpScreen::sizeHint() const
