@@ -28,7 +28,6 @@
 #include <QWidget>
 #include <QString>
 #include <QLabel>
-#include <QTimer>
 #include <QSizePolicy>
 #include <QSize>
 
@@ -57,11 +56,11 @@ class ArpScreen : public QWidget
   Q_OBJECT
 
   private:
-    QTimer *timer;
     int grooveTick, grooveVelocity, grooveLength;
     QString pattern;
     int pattern_updated, currentIndex;
     bool isMuted;
+    bool needsRedraw;
 
   protected:
     virtual void paintEvent(QPaintEvent *);
@@ -84,6 +83,7 @@ class ArpScreen : public QWidget
     void updateScreen(int p_index);
     void newGrooveValues(int tick, int vel, int length);
     void setMuted(bool on);
+    void updateDraw();
 };
 
 #endif
