@@ -42,16 +42,23 @@
 
 
 
-ArpWidget::ArpWidget(MidiArp *p_midiWorker, int portCount, bool compactStyle,
-    bool mutedAdd, QWidget *parent):
-    QWidget(parent), midiWorker(p_midiWorker), modified(false)
+ArpWidget::ArpWidget(MidiArp *p_midiWorker, GlobStore *p_globStore,
+    int portCount, bool compactStyle,
+    bool mutedAdd, bool inOutVisible, const QString& name, QWidget *parent):
+    QWidget(parent), midiWorker(p_midiWorker),
+    globStore(p_globStore), modified(false)
 {
     int l1;
 
-    parStore = new ParStore();
-
     midiControl = new MidiControl(this);
 
+    parStore = new ParStore(globStore, name, this);
+    midiControl->addMidiLearnMenu("Restore_"+name, parStore->topButton, 2);
+    connect(parStore, SIGNAL(store(int, bool)),
+             this, SLOT(storeParams(int, bool)));
+    connect(parStore, SIGNAL(restore(int)),
+             this, SLOT(restoreParams(int)));
+
     manageBox = new ManageBox("Arp:", false, this);
 
     // Input group box on left side
@@ -123,12 +130,6 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, int portCount, bool compactStyle,
     // Output group box on right side
     QGroupBox *portBox = new QGroupBox(tr("Output"), this);
 
-    QLabel *muteLabel = new QLabel(tr("&Mute"),portBox);
-    muteOut = new QCheckBox(this);
-    connect(muteOut, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
-    muteLabel->setBuddy(muteOut);
-    midiControl->addMidiLearnMenu("MuteToggle", muteOut, 0);
-
     QLabel *portLabel = new QLabel(tr("&Port"), portBox);
     portOut = new QComboBox(portBox);
     portLabel->setBuddy(portOut);
@@ -143,18 +144,24 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, int portCount, bool compactStyle,
             SLOT(updateChannelOut(int)));
 
     QGridLayout *portBoxLayout = new QGridLayout;
-    portBoxLayout->addWidget(muteLabel, 0, 0);
-    portBoxLayout->addWidget(muteOut, 0, 1);
-    portBoxLayout->addWidget(portLabel, 1, 0);
-    portBoxLayout->addWidget(portOut, 1, 1);
-    portBoxLayout->addWidget(channelLabel, 2, 0);
-    portBoxLayout->addWidget(channelOut, 2, 1);
+    portBoxLayout->addWidget(portLabel, 0, 0);
+    portBoxLayout->addWidget(portOut, 0, 1);
+    portBoxLayout->addWidget(channelLabel, 1, 0);
+    portBoxLayout->addWidget(channelOut, 1, 1);
     if (compactStyle) {
         portBoxLayout->setMargin(2);
         portBoxLayout->setSpacing(1);
     }
     portBox->setLayout(portBoxLayout);
 
+    hideInOutBoxAction = new QAction(tr("&Show/hide in-out settings"), this);
+    QToolButton *hideInOutBoxButton = new QToolButton(this);
+    hideInOutBoxAction->setCheckable(true);
+    hideInOutBoxAction->setChecked(inOutVisible);
+    hideInOutBoxButton->setDefaultAction(hideInOutBoxAction);
+    hideInOutBoxButton->setFixedSize(10, 80);
+    hideInOutBoxButton->setArrowType (Qt::ArrowType(0));
+    connect(hideInOutBoxAction, SIGNAL(toggled(bool)), this, SLOT(setInOutBoxVisible(bool)));
 
     // Layout for left/right placements of in/out group boxes
     QVBoxLayout *inOutBoxLayout = new QVBoxLayout();
@@ -162,6 +169,9 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, int portCount, bool compactStyle,
     inOutBoxLayout->addWidget(inBox);
     inOutBoxLayout->addWidget(portBox);
     inOutBoxLayout->addStretch();
+    inOutBox = new QWidget(this);
+    inOutBox->setLayout(inOutBoxLayout);
+    inOutBox->setVisible(inOutVisible);
 
     // group box for pattern setup
     QGroupBox *patternBox = new QGroupBox(tr("Pattern"), this);
@@ -195,12 +205,20 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, int portCount, bool compactStyle,
     loadPatternPresets();
     patternPresetBox->insertItems(0, patternNames);
     patternPresetBox->setCurrentIndex(0);
+    patternPresetBoxIndex = 0;
     patternPresetBox->setToolTip(tr("Pattern preset"));
     patternPresetBox->setMinimumContentsLength(20);
     connect(patternPresetBox, SIGNAL(activated(int)), this,
             SLOT(selectPatternPreset(int)));
     midiControl->addMidiLearnMenu("PresetSwitch", patternPresetBox, 1);
 
+    muteOut = new QPushButton(tr("&Mute"),this);
+    muteOut->setFont(QFont("Helvetica", 8));
+    muteOut->setMinimumSize(QSize(35,10));
+    muteOut->setCheckable(true);
+    connect(muteOut, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
+    midiControl->addMidiLearnMenu("MuteToggle", muteOut, 0);
+
     repeatPatternThroughChord = new QComboBox(patternBox);
     QStringList repeatPatternNames;
     repeatPatternNames << tr("Static") << tr("Up") << tr("Down");
@@ -245,6 +263,7 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, int portCount, bool compactStyle,
         modeLayout->setSpacing(1);
     }
 
+    modeLayout->addWidget(muteOut);
     modeLayout->addWidget(repeatPatternThroughChord);
     modeLayout->addWidget(triggerMode);
     modeLayout->addWidget(latchModeButton);
@@ -258,6 +277,7 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, int portCount, bool compactStyle,
             tr("0..9  note played on keyboard, 0 is lowest\n"
             "( ) numbers in parenthesis are stacked to chords\n"
             "  + = -  octave up/reset/down\n"
+            "  t = g  semitone up/reset/down\n"
             " < . > tempo up/reset/down\n"
             "  d h  note length up/down\n"
             "  / \\  velocity up/down\n"
@@ -348,14 +368,17 @@ ArpWidget::ArpWidget(MidiArp *p_midiWorker, int portCount, bool compactStyle,
     widgetLayout->addWidget(patternBox, 0, 0);
     widgetLayout->addWidget(randomBox, 1, 0);
     widgetLayout->addWidget(envelopeBox, 2, 0);
-    widgetLayout->addLayout(inOutBoxLayout, 0, 1, 3, 1);
+    widgetLayout->addWidget(hideInOutBoxButton, 0, 1);
+    widgetLayout->addWidget(inOutBox, 0, 2, 3, 1);
     widgetLayout->setRowStretch(3, 1);
     widgetLayout->setColumnStretch(0, 5);
     setLayout(widgetLayout);
+    needsGUIUpdate=false;
 }
 
 ArpWidget::~ArpWidget()
 {
+    delete parStore;
 }
 
 MidiArp *ArpWidget::getMidiWorker()
@@ -405,6 +428,7 @@ void ArpWidget::writeData(QXmlStreamWriter& xml)
 {
     xml.writeStartElement(manageBox->name.left(3));
     xml.writeAttribute("name", manageBox->name.mid(manageBox->name.indexOf(':') + 1));
+    xml.writeAttribute("inOutVisible", QString::number(inOutBox->isVisible()));
         xml.writeStartElement("pattern");
             xml.writeTextElement("pattern", midiWorker->pattern);
             xml.writeTextElement("repeatMode", QString::number(
@@ -793,6 +817,12 @@ void ArpWidget::setEnvelopeVisible(bool on)
     releaseTime->setVisible(on);
 }
 
+void ArpWidget::setInOutBoxVisible(bool on)
+{
+    inOutBox->setVisible(on);
+    modified=true;
+}
+
 void ArpWidget::setMuted(bool on)
 {
     midiWorker->setMuted(on);
@@ -877,7 +907,7 @@ void ArpWidget::restoreParams(int ix)
     randomLength->setValue(parStore->list.at(ix).rndLen);
     randomVelocity->setValue(parStore->list.at(ix).rndVel);
 
-    muteOut->setChecked(parStore->list.at(ix).muteOut);
+    //muteOut->setChecked(parStore->list.at(ix).muteOut);
     chIn->setCurrentIndex(parStore->list.at(ix).chIn);
     updateChIn(parStore->list.at(ix).chIn);
     channelOut->setCurrentIndex(parStore->list.at(ix).channelOut);
@@ -903,26 +933,56 @@ void ArpWidget::handleController(int ccnumber, int channel, int value)
             switch (cclist.at(l2).ID) {
                 case 0: if (min == max) {
                             if (value == max) {
-                                m = muteOut->isChecked();
-                                muteOut->setChecked(!m);
+                                m = midiWorker->isMuted;
+                                midiWorker->setMuted(!m);
                             }
                         }
                         else {
                             if (value == max) {
-                                muteOut->setChecked(false);
+                                midiWorker->setMuted(false);
                             }
                             if (value == min) {
-                                muteOut->setChecked(true);
+                                midiWorker->setMuted(true);
                             }
                         }
                 break;
                 case 1:
                         sval = min + ((double)value * (max - min) / 127);
-                        selectPatternPreset(sval);
+                        patternPresetBoxIndex = sval;
                 break;
+                case 2:
+                        sval = min + ((double)value * (max - min) / 127);
+                        if ((sval < parStore->list.count())
+                                && (sval != parStore->activeStore)
+                                && (sval != parStore->currentRequest)) {
+                            parStore->requestDispState(sval, 2);
+                            parStore->restoreRequest = sval;
+                            parStore->restoreRunOnce = (parStore->jumpToList.at(sval) > -2);
+                        }
+                        else return;
+                break;
+
                 default:
                 break;
             }
+            needsGUIUpdate = true;
         }
     }
 }
+
+void ArpWidget::updateDisplay()
+{
+    int frame = (getGrooveIndex() > 0) ? getGrooveIndex() - 1 : 0;
+
+    parStore->updateDisplay(frame, false);
+
+    screen->updateDraw();
+    midiControl->update();
+
+    if (!needsGUIUpdate) return;
+
+    muteOut->setChecked(midiWorker->isMuted);
+    if (patternPresetBoxIndex != patternPresetBox->currentIndex())
+        selectPatternPreset(patternPresetBoxIndex);
+
+}
