@@ -28,6 +28,7 @@
 #include <QString>
 #include <QTextStream>
 #include <QToolButton>
+#include <QPushButton>
 #include <QAction>
 #include <QComboBox>
 #include <QGroupBox>
@@ -38,6 +39,7 @@
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
 
+#include "globstore.h"
 #include "midiarp.h"
 #include "slider.h"
 #include "arpscreen.h"
@@ -70,17 +72,21 @@ class ArpWidget : public QWidget
     QComboBox *triggerMode;
     QComboBox *patternPresetBox;
     QGroupBox *inputFilterBox, *randomBox, *envelopeBox;
+    QWidget *inOutBox;
     QToolButton *textEditButton, *textStoreButton, *textRemoveButton;
     QToolButton *latchModeButton;
     QAction *textEditAction, *textStoreAction, *textRemoveAction;
     QAction *latchModeAction;
     MidiArp *midiWorker;
+    GlobStore *globStore;
     QLineEdit *patternText;
     Slider *randomVelocity, *randomTick, *randomLength;
     Slider *attackTime, *releaseTime;
     bool modified;      /**< @brief Is set to True if unsaved parameter modifications exist */
-    void loadPatternPresets();
+    bool needsGUIUpdate;
+    int patternPresetBoxIndex;
 
+    void loadPatternPresets();
 /* PUBLIC MEMBERS */
   public:
 /*!
@@ -93,8 +99,10 @@ class ArpWidget : public QWidget
  * @param mutedAdd If set to True, the module will be added in muted state
  * @param parent The parent widget of this module, i.e. MainWindow
  */
-    ArpWidget(MidiArp *p_midiWorker, int portCount, bool compactStyle,
-            bool mutedAdd = false, QWidget* parent=0);
+    ArpWidget(MidiArp *p_midiWorker, GlobStore *p_globStore,
+            int portCount, bool compactStyle,
+            bool mutedAdd = false, bool inOutVisible = true,
+            const QString& name = "", QWidget* parent=0);
     ~ArpWidget();
 
     ParStore *parStore;
@@ -103,7 +111,8 @@ class ArpWidget : public QWidget
     ManageBox *manageBox;
 
     QStringList patternPresets, patternNames;
-    QCheckBox *muteOut;
+    QPushButton *muteOut;
+    QAction* hideInOutBoxAction;
 
     void setChIn(int value);
     void setIndexIn(int index, int value);
@@ -125,20 +134,6 @@ class ArpWidget : public QWidget
 */
     void writeData(QXmlStreamWriter& xml);
 /*!
-* @brief This function stores some module parameters in a parameter
-* list object
-*
-* @param Position index in the parameter list
-*/
-    void storeParams(int ix, bool empty = false);
-/*!
-* @brief This function restores some module parameters from the parameter
-* list object
-*
-* @param Position index in the parameter list
-*/
-    void restoreParams(int ix);
-/*!
 * @brief This function returns the MidiArp instance associated with this GUI
 * Widget.
 * @return MidiArp instance associated with this GUI
@@ -205,11 +200,14 @@ class ArpWidget : public QWidget
     void setRandomVisible(bool on);
     void setEnvelopeVisible(bool on);
     void setInputFilterVisible(bool on);
+    void setInOutBoxVisible(bool on);
 
     void updateChIn(int value);
     void updateIndexIn(int value);
     void updateRangeIn(int value);
 
+    void updateDisplay();
+
  /*! @brief Slot for ArpWidget::latchModeAction.
   * Will cause notes remaining in MidiArp::latchBuffer until new
   * stakato note received */
@@ -240,6 +238,20 @@ class ArpWidget : public QWidget
 *
 */
     void setMuted(bool on);
+/*!
+* @brief This function stores some module parameters in a parameter
+* list object
+*
+* @param Position index in the parameter list
+*/
+    void storeParams(int ix, bool empty = false);
+/*!
+* @brief This function restores some module parameters from the parameter
+* list object
+*
+* @param Position index in the parameter list
+*/
+    void restoreParams(int ix);
     void handleController(int ccnumber, int channel, int value);
     int getGrooveIndex() { return midiWorker->getGrooveIndex(); }
 };
