@@ -32,6 +32,11 @@
 Engine::Engine(GlobStore *p_globStore, GrooveWidget *p_grooveWidget, int p_portCount, bool p_alsamidi, QWidget *parent) : QThread(parent), modified(false)
 {
     ready = false;
+
+    logEventBuffer.resize(128);
+    logTickBuffer.resize(128);
+    logEventCount = 0;
+
     globStoreWidget = p_globStore;
     connect(globStoreWidget->midiControl, SIGNAL(setMidiLearn(int, int, int)),
             this, SLOT(setMidiLearn(int, int, int)));
@@ -68,23 +73,18 @@ Engine::Engine(GlobStore *p_globStore, GrooveWidget *p_grooveWidget, int p_portC
     useMidiClock = false;
     currentTick = 0;
     requestTick = 0;
+    tempo = 120;
 
     restoreRequest = -1;
     restoreModIx = 0;
     restoreModType = 'X';
     restoreModWindowIndex = -1;
     restoreTick = -1;
-
-    // we happen to need these QSignals for triggering display updates
-    // and global parameter restores from within the echo callback.
-    // Direct calls to the module widgets were shown to
-    // cause frequent segfaults.
-    connect(this, SIGNAL(restoreSig(int)), this, SLOT(restore(int)));
-    connect(this, SIGNAL(indicPercentSig(int)), this, SLOT(indicPercent(int)));
-    connect(this, SIGNAL(updateCursorSig(QChar, int, int))
-            , this, SLOT(updateCursor(QChar, int, int)));
+    schedRestoreLocation = -1;
 
     resetTicks(0);
+    dispTimer = new MTimer();
+    connect(dispTimer, SIGNAL(timeout()), this, SLOT(updateDisplay()));
     ready = true;
 }
 
@@ -321,31 +321,47 @@ int Engine::moduleWindowCount()
     return(moduleWindowList.count());
 }
 
+void Engine::renameDock(const QString& name, int parentDockID, int widgetID)
+{
+    moduleWindow(parentDockID)->setWindowTitle(name);
+    if (name.startsWith('S')) seqWidget(widgetID)->parStore->topButton->setText(name);
+    if (name.startsWith('L')) lfoWidget(widgetID)->parStore->topButton->setText(name);
+    if (name.startsWith('A')) arpWidget(widgetID)->parStore->topButton->setText(name);
+    setModified(true);
+}
+
+
 void Engine::updateIDs(int curID)
 {
     int l1, tempDockID;
     for (l1 = 0; l1 < arpWidgetCount(); l1++) {
         arpWidget(l1)->manageBox->ID = l1;
+        arpWidget(l1)->midiControl->ID = l1;
         arpWidget(l1)->setProperty("widgetID", l1);
         tempDockID = arpWidget(l1)->manageBox->parentDockID;
         if (tempDockID > curID) {
             arpWidget(l1)->manageBox->parentDockID = tempDockID - 1;
+            arpWidget(l1)->midiControl->parentDockID = tempDockID - 1;
         }
     }
     for (l1 = 0; l1 < lfoWidgetCount(); l1++) {
         lfoWidget(l1)->manageBox->ID = l1;
+        lfoWidget(l1)->midiControl->ID = l1;
         lfoWidget(l1)->setProperty("widgetID", l1);
         tempDockID = lfoWidget(l1)->manageBox->parentDockID;
         if (tempDockID > curID) {
             lfoWidget(l1)->manageBox->parentDockID = tempDockID - 1;
+            lfoWidget(l1)->midiControl->parentDockID = tempDockID - 1;
         }
     }
     for (l1 = 0; l1 < seqWidgetCount(); l1++) {
         seqWidget(l1)->manageBox->ID = l1;
+        seqWidget(l1)->midiControl->ID = l1;
         seqWidget(l1)->setProperty("widgetID", l1);
         tempDockID = seqWidget(l1)->manageBox->parentDockID;
         if (tempDockID > curID) {
             seqWidget(l1)->manageBox->parentDockID = tempDockID - 1;
+            seqWidget(l1)->midiControl->parentDockID = tempDockID - 1;
         }
     }
 }
@@ -367,6 +383,8 @@ void Engine::setCompactStyle(bool on)
 
 bool Engine::isModified()
 {
+    if (!moduleWindowCount()) return false;
+
     bool arpmodified = false;
     bool lfomodified = false;
     bool seqmodified = false;
@@ -388,7 +406,8 @@ bool Engine::isModified()
             break;
         }
 
-    return modified || arpmodified || lfomodified || seqmodified;
+    return (modified || arpmodified || lfomodified || seqmodified
+            || globStoreWidget->isModified());
 }
 
 void Engine::setModified(bool m)
@@ -403,6 +422,8 @@ void Engine::setModified(bool m)
 
     for (int l1 = 0; l1 < seqWidgetCount(); l1++)
         seqWidget(l1)->setModified(m);
+
+    globStoreWidget->setModified(m);
 }
 
 /* All following functions are the core engine of QMidiArp. They need to
@@ -424,16 +445,16 @@ int Engine::getClientId()
 
 void Engine::setStatus(bool on)
 {
-    if (moduleWindowCount()) {
-        if (!on) {
-            for (int l1 = 0; l1 < midiArpCount(); l1++) {
-                midiArp(l1)->clearNoteBuffer();
-            }
+    int l1;
+    if (!moduleWindowCount()) return;
+    if (!on) {
+        for (l1 = 0; l1 < midiArpCount(); l1++) {
+            midiArp(l1)->clearNoteBuffer();
         }
-        status = on;
-        resetTicks(0);
-        driver->setTransportStatus(on);
     }
+    status = on;
+    resetTicks(0);
+    driver->setTransportStatus(on);
 }
 
 void Engine::tick_callback(void * context, bool echo_from_trig)
@@ -444,7 +465,6 @@ void Engine::tick_callback(void * context, bool echo_from_trig)
 void Engine::echoCallback(bool echo_from_trig)
 {
     int l1, l2;
-    QVector<int> note, velocity;
     int tick = driver->getCurrentTick();
     int note_tick = 0;
     int length;
@@ -457,9 +477,6 @@ void Engine::echoCallback(bool echo_from_trig)
 
     currentTick = tick;
 
-    note.clear();
-    velocity.clear();
-
         //~ printf("       tick %d     ",tick);
         //~ printf("nextMinLfoTick %d  ",nextMinLfoTick);
         //~ printf("nextMinSeqTick %d  ",nextMinSeqTick);
@@ -476,27 +493,26 @@ void Engine::echoCallback(bool echo_from_trig)
                     outEv.data = midiLfo(l1)->ccnumber;
                     outEv.channel = midiLfo(l1)->channelOut;
                     frameptr = midiLfo(l1)->getFramePtr();
-                    emit updateCursorSig('L', l1, frameptr);
-                    midiLfo(l1)->getNextFrame(&lfoData, tick);
+                    lfoWidget(l1)->cursor->updatePosition(frameptr);
+                    midiLfo(l1)->getNextFrame(tick);
                     outport = midiLfo(l1)->portOut;
-                    if (!midiLfo(l1)->isMuted) {
-                        l2 = 0;
-                        while (lfoData.at(l2).value > -1) {
-                            if (!lfoData.at(l2).muted) {
-                                outEv.value = lfoData.at(l2).value;
-                                driver->sendMidiEvent(outEv, lfoData.at(l2).tick
-                                    , outport);
-                            }
-                            l2++;
+                    l2 = 0;
+                    while (midiLfo(l1)->frame.at(l2).value > -1) {
+                        if (!midiLfo(l1)->frame.at(l2).muted && !midiLfo(l1)->isMuted) {
+                            outEv.value = midiLfo(l1)->frame.at(l2).value;
+                            driver->sendMidiEvent(outEv, midiLfo(l1)->frame.at(l2).tick
+                                , outport);
                         }
+                        l2++;
                     }
+                    frameptr = midiLfo(l1)->getFramePtr();
+                    percent = frameptr * 100 / (midiLfo(l1)->nPoints);
+                    lfoWidget(l1)->parStore->ndc->updatePercent(percent);
                     if ((restoreModType == 'L') && (l1 == restoreModIx)
                             && (!globStoreWidget->timeModeBox->currentIndex())) {
-                        frameptr = midiLfo(l1)->getFramePtr();
-                        percent = frameptr * 100 / (midiLfo(l1)->nPoints);
-                        emit indicPercentSig(percent);
+                        globStoreWidget->indicator->updatePercent(percent);
                         if (!frameptr && restoreFlag) {
-                            restoreTick = lfoData.last().tick;
+                            restoreTick = midiLfo(l1)->frame.at(l2).tick;
                             restoreFlag = false;
                         }
                     }
@@ -520,7 +536,8 @@ void Engine::echoCallback(bool echo_from_trig)
                     outEv.type = EV_NOTEON;
                     outEv.value = midiSeq(l1)->vel;
                     outEv.channel = midiSeq(l1)->channelOut;
-                    emit updateCursorSig('S', l1, midiSeq(l1)->getCurrentIndex());
+                    seqWidget(l1)->cursor->updatePosition(midiSeq(l1)->getCurrentIndex());
+
                     midiSeq(l1)->getNextNote(&seqSample, tick);
                     length = midiSeq(l1)->notelength;
                     outport = midiSeq(l1)->portOut;
@@ -528,11 +545,12 @@ void Engine::echoCallback(bool echo_from_trig)
                         outEv.data = seqSample.value;
                         driver->sendMidiEvent(outEv, seqSample.tick, outport, length);
                     }
-                    if ((restoreModType == 'S') && (l1 == restoreModIx)
-                          && (!globStoreWidget->timeModeBox->currentIndex())) {
                         frameptr = midiSeq(l1)->getCurrentIndex();
                         percent = frameptr * 100 / (midiSeq(l1)->nPoints);
-                        emit indicPercentSig(percent);
+                        seqWidget(l1)->parStore->ndc->updatePercent(percent);
+                    if ((restoreModType == 'S') && (l1 == restoreModIx)
+                          && (!globStoreWidget->timeModeBox->currentIndex())) {
+                        globStoreWidget->indicator->updatePercent(percent);
                         if (!frameptr && restoreFlag) {
                             restoreTick = midiSeq(l1)->nextTick;
                             restoreFlag = false;
@@ -558,29 +576,31 @@ void Engine::echoCallback(bool echo_from_trig)
                     outEv.channel = midiArp(l1)->channelOut;
                     midiArp(l1)->newRandomValues();
                     midiArp(l1)->prepareCurrentNote(tick + schedDelayTicks);
-                    note = midiArp(l1)->returnNote;
-                    velocity = midiArp(l1)->returnVelocity;
                     note_tick = midiArp(l1)->returnTick;
                     length = midiArp(l1)->returnLength * 4;
                     outport = midiArp(l1)->portOut;
                     isNew = midiArp(l1)->returnIsNew;
-                    emit updateCursorSig('A', l1, midiArp(l1)->getGrooveIndex());
-                    if (!velocity.isEmpty()) {
-                        if (isNew && velocity.at(0)) {
+                    arpWidget(l1)->screen->updateScreen(midiArp(l1)->getGrooveIndex());
+                    if (!midiArp(l1)->returnNote.isEmpty()) {
+                        if (isNew && midiArp(l1)->returnVelocity.at(0)) {
                             l2 = 0;
-                            while(note.at(l2) >= 0) {
-                                outEv.data = note.at(l2);
-                                outEv.value = velocity.at(l2);
+                            while(midiArp(l1)->returnNote.at(l2) >= 0) {
+                                outEv.data = midiArp(l1)->returnNote.at(l2);
+                                outEv.value = midiArp(l1)->returnVelocity.at(l2);
                                 driver->sendMidiEvent(outEv, note_tick, outport, length);
                                 l2++;
                             }
                         }
                     }
+                    frameptr = midiArp(l1)->getGrooveIndex() - 1;
+                    if (midiArp(l1)->nPoints)
+                        percent = frameptr * 100 / (midiArp(l1)->nPoints);
+                    else
+                        percent = 0;
+                    arpWidget(l1)->parStore->ndc->updatePercent(percent);
                     if ((restoreModType == 'A') && (l1 == restoreModIx)
                             && (!globStoreWidget->timeModeBox->currentIndex())) {
-                        frameptr = midiArp(l1)->getGrooveIndex() - 1;
-                        percent = frameptr * 100 / (midiArp(l1)->nPoints);
-                        emit indicPercentSig(percent);
+                        globStoreWidget->indicator->updatePercent(percent);
                         if (!frameptr && restoreFlag) {
                             restoreTick = note_tick;
                             restoreFlag = false;
@@ -600,7 +620,7 @@ void Engine::echoCallback(bool echo_from_trig)
 
     if (restoreFlag && (globStoreWidget->timeModeBox->currentIndex())) {
         percent = 100 * (currentTick - requestTick) / (restoreTick - requestTick);
-        emit indicPercentSig(percent);
+        globStoreWidget->indicator->updatePercent(percent);
     }
 
     if ((restoreTick > -1)
@@ -609,7 +629,10 @@ void Engine::echoCallback(bool echo_from_trig)
         && (!midiSeqCount() || (nextMinSeqTick >= restoreTick))) {
         restoreTick = -1;
         restoreFlag = false;
-        emit restoreSig(restoreRequest);
+        // TODO: At term the following should be restore() instead of
+        // schedRestore(). restore is not yet fit for realtime. But
+        // testing seems positive this way.
+        schedRestore(restoreRequest);
     }
 }
 
@@ -621,12 +644,16 @@ bool Engine::midi_event_received_callback(void * context, MidiEvent ev)
 bool Engine::eventCallback(MidiEvent inEv)
 {
     bool unmatched;
+    bool no_collision = false;
     int l1;
     unmatched = true;
     int tick = driver->getCurrentTick();
 
-    /* Does this cost time or other problems? The signal is sent to the LogWidget.*/
-    if (sendLogEvents) emit midiEventReceived(inEv, tick);
+    if (sendLogEvents) {
+        logEventBuffer.replace(logEventCount, inEv);
+        logTickBuffer.replace(logEventCount, tick);
+        logEventCount++;
+    }
 
     if (useMidiClock){
         if (inEv.type == EV_START) {
@@ -643,7 +670,8 @@ bool Engine::eventCallback(MidiEvent inEv)
         unmatched = midiLfo(l1)->handleEvent(inEv, tick);
         if (midiLfo(l1)->gotKbdTrig) {
             nextMinLfoTick = midiLfo(l1)->nextTick;
-            driver->requestEchoAt(nextMinLfoTick, true);
+            no_collision = driver->requestEchoAt(nextMinLfoTick, true);
+            if (!no_collision) midiLfo(l1)->gotKbdTrig = false;
         }
     }
     for (l1 = 0; l1 < midiSeqCount(); l1++) {
@@ -653,14 +681,16 @@ bool Engine::eventCallback(MidiEvent inEv)
         }
         if (midiSeq(l1)->gotKbdTrig) {
             nextMinSeqTick = midiSeq(l1)->nextTick;
-            driver->requestEchoAt(nextMinSeqTick, true);
+            no_collision = driver->requestEchoAt(nextMinSeqTick, true);
+            if (!no_collision) midiSeq(l1)->gotKbdTrig = false;
         }
     }
     for (l1 = 0; l1 < midiArpCount(); l1++) {
         unmatched = midiArp(l1)->handleEvent(inEv, tick, 1);
         if (midiArp(l1)->gotKbdTrig) {
             nextMinArpTick = midiArp(l1)->nextTick;
-            driver->requestEchoAt(nextMinArpTick, true);
+            no_collision = driver->requestEchoAt(nextMinArpTick, true);
+            if (!no_collision) midiArp(l1)->gotKbdTrig = false;
         }
     }
 
@@ -677,6 +707,18 @@ bool Engine::eventCallback(MidiEvent inEv)
     return unmatched;
 }
 
+void Engine::showAllIOPanels(bool on)
+{
+    int l1;
+
+    for (l1 = 0; l1 < arpWidgetCount(); l1++)
+        arpWidget(l1)->hideInOutBoxAction->setChecked(on);
+    for (l1 = 0; l1 < lfoWidgetCount(); l1++)
+        lfoWidget(l1)->hideInOutBoxAction->setChecked(on);
+    for (l1 = 0; l1 < seqWidgetCount(); l1++)
+        seqWidget(l1)->hideInOutBoxAction->setChecked(on);
+}
+
 void Engine::sendController(int ccnumber, int channel, int value)
 {
     int l1;
@@ -695,30 +737,31 @@ void Engine::sendController(int ccnumber, int channel, int value)
 void Engine::learnController(int ccnumber, int channel)
 {
     if (midiLearnWindowID == -1) {
-        grooveWidget->midiControl->appendMidiCC(midiLearnID,
+        grooveWidget->midiControl->requestAppendMidiCC(midiLearnID,
                 ccnumber, channel, -100, 100);
         midiLearnFlag = false;
         return;
         }
 
     if (midiLearnWindowID == -2) {
-        globStoreWidget->midiControl->appendMidiCC(midiLearnID,
+        globStoreWidget->midiControl->requestAppendMidiCC(midiLearnID,
                 ccnumber, channel, 0, 127);
         midiLearnFlag = false;
         return;
         }
 
     int min = (midiLearnID) ? 0 : 127; //if control is toggle min=max
-    if (moduleWindow(midiLearnWindowID)->objectName().startsWith("Arp")) {
-        arpWidget(midiLearnModuleID)->midiControl->appendMidiCC(midiLearnID,
+
+    if (midiLearnWindowID == 1) {
+        arpWidget(midiLearnModuleID)->midiControl->requestAppendMidiCC(midiLearnID,
                 ccnumber, channel, min, 127);
     }
-    if (moduleWindow(midiLearnWindowID)->objectName().startsWith("LFO")) {
-        lfoWidget(midiLearnModuleID)->midiControl->appendMidiCC(midiLearnID,
+    if (midiLearnWindowID == 2) {
+        lfoWidget(midiLearnModuleID)->midiControl->requestAppendMidiCC(midiLearnID,
                 ccnumber, channel, min, 127);
     }
-    if (moduleWindow(midiLearnWindowID)->objectName().startsWith("Seq")) {
-        seqWidget(midiLearnModuleID)->midiControl->appendMidiCC(midiLearnID,
+    if (midiLearnWindowID == 3) {
+        seqWidget(midiLearnModuleID)->midiControl->requestAppendMidiCC(midiLearnID,
                 ccnumber, channel, min, 127);
     }
 
@@ -758,6 +801,13 @@ void Engine::setUseMidiClock(bool on)
     setStatus(false);
     driver->setUseMidiClock(on);
     useMidiClock = on;
+    modified = true;
+}
+
+void Engine::setUseJackTransport(bool on)
+{
+    driver->setUseJackTransport(on);
+    modified = true;
 }
 
 void Engine::setMidiLearn(int moduleWindowID, int moduleID, int controlID)
@@ -769,6 +819,12 @@ void Engine::setMidiLearn(int moduleWindowID, int moduleID, int controlID)
     else {
         midiLearnFlag = true;
         midiLearnWindowID = moduleWindowID;
+        if (midiLearnWindowID >= 0) {
+            QChar test = moduleWindow(midiLearnWindowID)->objectName().at(0);
+            if (test == 'A') midiLearnWindowID = 1;
+            if (test == 'L') midiLearnWindowID = 2;
+            if (test == 'S') midiLearnWindowID = 3;
+        }
         midiLearnModuleID = moduleID;
         midiLearnID = controlID;
     }
@@ -777,12 +833,14 @@ void Engine::setMidiLearn(int moduleWindowID, int moduleID, int controlID)
 void Engine::setTempo(int bpm)
 {
     driver->setTempo(bpm);
+    tempo=bpm;
     modified = true;
 }
 
 void Engine::setSendLogEvents(bool on)
 {
     sendLogEvents = on;
+    modified = true;
 }
 
 void Engine::tr_state_cb(bool on, void *context)
@@ -794,33 +852,10 @@ void Engine::tr_state_cb(bool on, void *context)
     }
 }
 
-void Engine::indicPercent(int p)
-{
-    globStoreWidget->indicator->updatePercent(p);
-}
-
-void Engine::updateCursor(QChar modtype, int ix, int pos)
-{
-    switch (modtype.toLatin1()) {
-
-        case 'S':
-            seqWidget(ix)->screen->updateScreen(pos);
-        break;
-
-        case 'L':
-            lfoWidget(ix)->updateScreen(pos);
-        break;
-
-        case 'A':
-            arpWidget(ix)->screen->updateScreen(pos);
-        break;
-
-        default: ;
-    }
-}
-void Engine::globStore(int ix)
+void Engine::store(int ix)
 {
     int l1;
+
     for (l1 = 0; l1 < arpWidgetCount(); l1++) {
         arpWidget(l1)->storeParams(ix);
     }
@@ -830,68 +865,61 @@ void Engine::globStore(int ix)
     for (l1 = 0; l1 < seqWidgetCount(); l1++) {
         seqWidget(l1)->storeParams(ix);
     }
-    globStoreWidget->setDispState(ix, 1);
 }
 
-void Engine::requestRestore(int windowIndex, int ix)
+void Engine::requestRestore(int ix)
 {
-    restoreModWindowIndex = windowIndex;
-
     if (status == false) {
         restore(ix);
+        return;
     }
-    else {
-        restoreRequest = ix;
-        globStoreWidget->setDispState(ix, 2, windowIndex);
-        if (globStoreWidget->timeModeBox->currentIndex()) {
-            requestTick = currentTick;
-            restoreTick = TPQN * (2 + globStoreWidget->switchAtBeatBox
-                ->currentIndex() + currentTick / TPQN);
-        }
+
+    restoreRequest = ix;
+
+    globStoreWidget->setDispState(ix, 2);
+    if (globStoreWidget->timeModeBox->currentIndex()) {
+        requestTick = currentTick;
+        restoreTick = TPQN * (2 + globStoreWidget->switchAtBeatBox
+            ->currentIndex() + currentTick / TPQN);
     }
 }
 
+void Engine::schedRestore(int ix)
+{
+    schedRestoreLocation = ix;
+}
+
 void Engine::restore(int ix)
 {
-    if (restoreModWindowIndex < 0) {
-        int l1;
-        for (l1 = 0; l1 < arpWidgetCount(); l1++) {
-            arpWidget(l1)->restoreParams(ix);
-        }
-        for (l1 = 0; l1 < lfoWidgetCount(); l1++) {
-            lfoWidget(l1)->restoreParams(ix);
-        }
-        for (l1 = 0; l1 < seqWidgetCount(); l1++) {
-            seqWidget(l1)->restoreParams(ix);
-        }
+    int l1;
+    for (l1 = 0; l1 < arpWidgetCount(); l1++) {
+        arpWidget(l1)->restoreParams(ix);
+        arpWidget(l1)->parStore->oldRestoreRequest = ix;
     }
-    else {
-        if (restoreModType == 'A') {
-            arpWidget(restoreModIx)->restoreParams(ix);
-        }
-        else if (restoreModType == 'L') {
-            lfoWidget(restoreModIx)->restoreParams(ix);
-        }
-        else if (restoreModType == 'S') {
-            seqWidget(restoreModIx)->restoreParams(ix);
-        }
+    for (l1 = 0; l1 < lfoWidgetCount(); l1++) {
+        lfoWidget(l1)->restoreParams(ix);
+        lfoWidget(l1)->parStore->oldRestoreRequest = ix;
+    }
+    for (l1 = 0; l1 < seqWidgetCount(); l1++) {
+        seqWidget(l1)->restoreParams(ix);
+        seqWidget(l1)->parStore->oldRestoreRequest = ix;
     }
-
-    globStoreWidget->setDispState(ix, 1, restoreModWindowIndex);
     restoreRequest = -1;
+
+    globStoreWidget->requestDispState(ix, 1);
 }
 
 void Engine::removeParStores(int ix)
 {
     int l1;
     for (l1 = 0; l1 < arpWidgetCount(); l1++) {
-        arpWidget(l1)->parStore->list.removeAt(ix);
+        arpWidget(l1)->parStore->removeLocation(ix);
     }
     for (l1 = 0; l1 < lfoWidgetCount(); l1++) {
-        lfoWidget(l1)->parStore->list.removeAt(ix);
+        lfoWidget(l1)->parStore->removeLocation(ix);
     }
     for (l1 = 0; l1 < seqWidgetCount(); l1++) {
-        seqWidget(l1)->parStore->list.removeAt(ix);
+        seqWidget(l1)->parStore->removeLocation(ix);
     }
 }
 
@@ -903,3 +931,51 @@ void Engine::updateGlobRestoreTimeModule(int windowIndex)
     restoreModIx = moduleWindowList.at(windowIndex)->widget()
                         ->property("widgetID").toInt();
 }
+
+void Engine::updateDisplay()
+{
+    int l1;
+    // The following is a test whether the display update loop is fast
+    // enough to restore the parameters in time. This cannot be done in
+    // realtime currently, since it requires reworking the restore routines
+    // entirely. Testing seems positive, but TODO.
+
+    if (schedRestoreLocation >= 0) {
+        restore(schedRestoreLocation);
+        schedRestoreLocation = -1;
+    }
+
+    for (l1 = 0; l1 < arpWidgetCount(); l1++) {
+        arpWidget(l1)->updateDisplay();
+    }
+    for (l1 = 0; l1 < lfoWidgetCount(); l1++) {
+        lfoWidget(l1)->updateDisplay();
+    }
+    for (l1 = 0; l1 < seqWidgetCount(); l1++) {
+        seqWidget(l1)->updateDisplay();
+    }
+
+    globStoreWidget->updateDisplay();
+    grooveWidget->updateDisplay();
+
+    if ((sendLogEvents) && (logEventCount)) {
+        for (l1 = 0; l1 < logEventCount; l1++) {
+            emit midiEventReceived(logEventBuffer.at(l1), logTickBuffer.at(l1));
+        }
+        logEventCount = 0;
+    }
+}
+
+MTimer::MTimer()
+{
+    start();
+}
+
+void MTimer::run() {
+
+    while(true) {
+        // set fixed to 5000us
+        usleep(5000);
+        emit timeout();
+    }
+}
