@@ -42,9 +42,7 @@ LfoScreen::LfoScreen(QWidget* parent) : QWidget (parent)
     grooveVelocity = 0;
     grooveLength = 0;
     isMuted = false;
-    timer = new QTimer(this);
-    timer->setSingleShot(true);
-    connect(timer, SIGNAL(timeout()), this, SLOT(update()));
+    needsRedraw = false;
 }
 
 LfoScreen::~LfoScreen()
@@ -92,7 +90,6 @@ void LfoScreen::paintEvent(QPaintEvent*)
     p.setViewport(0, 0, w, h);
     p.setWindow(0, 0, w, h);
     p.setPen(QColor(160, 20, 20));
-    p.drawRect(0, 0, w - 1, h - 1);
 
     //Beat separators
     for (l1 = 0; l1 < nsteps + 1; l1++) {
@@ -136,9 +133,10 @@ void LfoScreen::paintEvent(QPaintEvent*)
     pen.setWidth(notestreak_thick);
     p.setPen(pen);
     grooveTmp = (beatRes < 32) ? grooveTick : 0;
-    for (l1 = 0; l1 < npoints; l1++) {
+    l1 = 0;
+    while (l1 < npoints) {
 
-        x = (l1 + .01 * (double)grooveTmp * (l1 % 2)) * nsteps * xscale / npoints;
+        x = (l1 + .01 * (double)grooveTmp * (l1 % 2)) * (int)nsteps * xscale / npoints;
         ypos = yscale - yscale * p_data.at(l1).value / 128
                         + LFOSCR_VMARG;
         xpos = LFOSCR_HMARG + x + pen.width() / 2;
@@ -150,18 +148,12 @@ void LfoScreen::paintEvent(QPaintEvent*)
         }
         p.setPen(pen);
         p.drawLine(xpos, ypos,
-                        xpos + (xscale / beatRes) - pen.width(), ypos);
+                        xpos + (xscale / beatRes)
+                        - (pen.width()/(2+npoints/(TPQN*8))), ypos);
+        l1++;
+        l1+=npoints/(TPQN*4);
     }
 
-    // Cursor
-    pen.setWidth(notestreak_thick * 2);
-    pen.setColor(QColor(200, 180, 70));
-    p.setPen(pen);
-    x = (currentIndex + .01 * (double)grooveTmp * (currentIndex % 2)) * xscale * (int)nsteps / npoints;
-    xpos = LFOSCR_HMARG + x + pen.width() / 2;
-    p.drawLine(xpos, h - 2,
-                    xpos + (xscale / beatRes) - pen.width(), h - 2);
-
     //Horizontal separators and numbers
     p.setPen(QColor(180, 120, 40));
     for (l1 = 0; l1 < 3; l1++) {
@@ -173,23 +165,23 @@ void LfoScreen::paintEvent(QPaintEvent*)
 
 }
 
-
-void LfoScreen::updateScreen(const QVector<Sample>& data)
+void LfoScreen::updateData(const QVector<Sample>& data)
 {
     p_data = data;
-    update();
+    needsRedraw = true;
 }
 
-void LfoScreen::updateScreen(int p_index)
+void LfoScreen::updateDraw()
 {
-    currentIndex = p_index;
+    if (!needsRedraw) return;
+    needsRedraw = false;
     update();
 }
 
 void LfoScreen::setMuted(bool on)
 {
     isMuted = on;
-    update();
+    needsRedraw = true;
 }
 
 void LfoScreen::mouseMoveEvent(QMouseEvent *event)
@@ -253,7 +245,7 @@ void LfoScreen::newGrooveValues(int tick, int vel, int length)
     grooveTick = tick;
     grooveVelocity = vel;
     grooveLength = length;
-    if (!timer->isActive()) timer->start(50);
+    needsRedraw = true;
 }
 
 QSize LfoScreen::sizeHint() const
