@@ -29,7 +29,6 @@
 #include <QString>
 #include <QLabel>
 #include <QMouseEvent>
-#include <QTimer>
 #include <QWheelEvent>
 #include <QSizePolicy>
 #include <QSize>
@@ -46,10 +45,8 @@
  *
  * LfoScreen is created and embedded by SeqWidget. The painter callback
  * produces a streak map of a sequence as a piano roll display. The
- * display is updated by calling LfoScreen::updateScreen() with the
- * Sample vector as argument. A cursor is placed at the corresponding
- * vector index by calling LfoScreen::updateScreen() with the integer
- * current index as an overloaded member.
+ * display is updated by calling LfoScreen::updateData() with the
+ * Sample vector as argument followed by updateDraw().
  * LfoScreen emits mouse events corresponding to the Qt mousePressed()
  * and mouseMoved() events. The mouse position is transferred as a
  * double from 0 ... 1.0 representing the relative mouse position on the
@@ -60,7 +57,6 @@ class LfoScreen : public QWidget
   Q_OBJECT
 
   private:
-    QTimer *timer;
     QVector<Sample> p_data, data;
     int grooveTick, grooveVelocity, grooveLength;
     int mouseX, mouseY, mouseW;
@@ -70,6 +66,7 @@ class LfoScreen : public QWidget
     int clip(int value, int min, int max, bool *outOfRange);
     bool recordMode;
     bool isMuted;
+    bool needsRedraw;
 
   protected:
     virtual void paintEvent(QPaintEvent *);
@@ -86,14 +83,14 @@ class LfoScreen : public QWidget
     void mouseWheel(int);
 
   public slots:
-    void updateScreen(const QVector<Sample>& data);
-    void updateScreen(int p_index);
+    void updateData(const QVector<Sample>& data);
     void mouseMoveEvent(QMouseEvent* event);
     void mousePressEvent(QMouseEvent* event);
     void setRecord(bool on);
     void wheelEvent(QWheelEvent* event);
     void newGrooveValues(int tick, int vel, int length);
     void setMuted(bool on);
+    void updateDraw();
 };
 
 #endif
