@@ -43,16 +43,23 @@
 #include "config.h"
 
 
-LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
-    bool mutedAdd, QWidget *parent):
-    QWidget(parent), midiWorker(p_midiWorker), modified(false)
+LfoWidget::LfoWidget(MidiLfo *p_midiWorker, GlobStore *p_globStore,
+    int portCount, bool compactStyle,
+    bool mutedAdd, bool inOutVisible, const QString& name, QWidget *parent):
+    QWidget(parent), midiWorker(p_midiWorker),
+    globStore(p_globStore), modified(false)
 {
     int l1;
 
-    parStore = new ParStore();
-
     midiControl = new MidiControl(this);
 
+    parStore = new ParStore(globStore, name, this);
+    midiControl->addMidiLearnMenu("Restore_"+name, parStore->topButton, 9);
+    connect(parStore, SIGNAL(store(int, bool)),
+             this, SLOT(storeParams(int, bool)));
+    connect(parStore, SIGNAL(restore(int)),
+             this, SLOT(restoreParams(int)));
+
     manageBox = new ManageBox("LFO:", true, this);
 
     // Input group box on right top
@@ -128,13 +135,6 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     // Output group box on right side
     QGroupBox *portBox = new QGroupBox(tr("Output"), this);
 
-    QLabel *muteLabel = new QLabel(tr("&Mute"),portBox);
-    muteOut = new QCheckBox(this);
-    connect(muteOut, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
-    muteLabel->setBuddy(muteOut);
-    midiControl->addMidiLearnMenu("MuteToggle", muteOut, 0);
-
-
     QLabel *ccnumberLabel = new QLabel(tr("MIDI &CC#"), portBox);
     ccnumberBox = new QSpinBox(portBox);
     ccnumberLabel->setBuddy(ccnumberBox);
@@ -160,14 +160,12 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
 
 
     QGridLayout *portBoxLayout = new QGridLayout;
-    portBoxLayout->addWidget(muteLabel, 0, 0);
-    portBoxLayout->addWidget(muteOut, 0, 1);
-    portBoxLayout->addWidget(ccnumberLabel, 1, 0);
-    portBoxLayout->addWidget(ccnumberBox, 1, 1);
-    portBoxLayout->addWidget(portLabel, 2, 0);
-    portBoxLayout->addWidget(portOut, 2, 1);
-    portBoxLayout->addWidget(channelLabel, 3, 0);
-    portBoxLayout->addWidget(channelOut, 3, 1);
+    portBoxLayout->addWidget(ccnumberLabel, 0, 0);
+    portBoxLayout->addWidget(ccnumberBox, 0, 1);
+    portBoxLayout->addWidget(portLabel, 1, 0);
+    portBoxLayout->addWidget(portOut, 1, 1);
+    portBoxLayout->addWidget(channelLabel, 2, 0);
+    portBoxLayout->addWidget(channelOut, 2, 1);
 
     QVBoxLayout* outputLayout = new QVBoxLayout;
     outputLayout->addLayout(portBoxLayout);
@@ -178,11 +176,23 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
 
     portBox->setLayout(outputLayout);
 
+    hideInOutBoxAction = new QAction(tr("&Show/hide in-out settings"), this);
+    QToolButton *hideInOutBoxButton = new QToolButton(this);
+    hideInOutBoxAction->setCheckable(true);
+    hideInOutBoxAction->setChecked(inOutVisible);
+    hideInOutBoxButton->setDefaultAction(hideInOutBoxAction);
+    hideInOutBoxButton->setFixedSize(10, 80);
+    hideInOutBoxButton->setArrowType (Qt::ArrowType(0));
+    connect(hideInOutBoxAction, SIGNAL(toggled(bool)), this, SLOT(setInOutBoxVisible(bool)));
+
     QVBoxLayout *inOutBoxLayout = new QVBoxLayout;
     inOutBoxLayout->addWidget(manageBox);
     inOutBoxLayout->addWidget(inBox);
     inOutBoxLayout->addWidget(portBox);
     inOutBoxLayout->addStretch();
+    inOutBox = new QWidget(this);
+    inOutBox->setLayout(inOutBoxLayout);
+    inOutBox->setVisible(inOutVisible);
 
     // group box for wave setup
     QGroupBox *waveBox = new QGroupBox(tr("Wave"), this);
@@ -191,12 +201,16 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     screen->setToolTip(
         tr("Right button to mute points\nLeft button to draw custom wave\nWheel to change offset"));
     screen->setMinimumHeight(80);
+
     connect(screen, SIGNAL(mouseMoved(double, double, int)), this,
             SLOT(mouseMoved(double, double, int)));
     connect(screen, SIGNAL(mousePressed(double, double, int)), this,
             SLOT(mousePressed(double, double, int)));
     connect(screen, SIGNAL(mouseWheel(int)), this,
             SLOT(mouseWheel(int)));
+
+    cursor = new Cursor('L', this);
+
     QLabel *waveFormBoxLabel = new QLabel(tr("&Waveform"), waveBox);
     waveFormBox = new QComboBox(waveBox);
     waveFormBoxLabel->setBuddy(waveFormBox);
@@ -208,6 +222,7 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     waveFormBox->addItem(QIcon(lfowsquare_xpm),"");
     waveFormBox->addItem(QIcon(lfowcustm_xpm),"");
     waveFormBox->setCurrentIndex(0);
+    waveFormBoxIndex = 0;
     waveFormBox->setToolTip(tr("Waveform Basis"));
 
     connect(waveFormBox, SIGNAL(activated(int)), this,
@@ -225,6 +240,7 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
         << "4" << "5" << "6" << "7" << "8";
     freqBox->insertItems(0, names);
     freqBox->setCurrentIndex(6);
+    freqBoxIndex = 6;
     freqBox->setToolTip(
             tr("Frequency (cycles/beat): Number of wave cycles produced every beat"));
     freqBox->setMinimumContentsLength(3);
@@ -240,6 +256,7 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
     names << "1" << "2" << "4" << "8" << "16" << "32" << "64" << "96" << "192";
     resBox->insertItems(0, names);
     resBox->setCurrentIndex(4);
+    resBoxIndex = 4;
     resBox->setToolTip(
             tr("Resolution (events/beat): Number of events produced every beat"));
     resBox->setMinimumContentsLength(3);
@@ -255,6 +272,7 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
             << "12" << "16" << "24" << "32" ;
     sizeBox->insertItems(0, names);
     sizeBox->setCurrentIndex(0);
+    sizeBoxIndex = 0;
     sizeBox->setToolTip(tr("Length of LFO wave in beats"));
     sizeBox->setMinimumContentsLength(3);
     connect(sizeBox, SIGNAL(activated(int)), this,
@@ -272,6 +290,13 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
             SLOT(updateLoop(int)));
     midiControl->addMidiLearnMenu("LoopMode", loopBox, 8);
 
+    muteOut = new QPushButton(tr("&Mute"),this);
+    muteOut->setFont(QFont("Helvetica", 8));
+    muteOut->setMinimumSize(QSize(35,10));
+    muteOut->setCheckable(true);
+    connect(muteOut, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
+    midiControl->addMidiLearnMenu("MuteToggle", muteOut, 0);
+
 
     QLabel *recordButtonLabel = new QLabel(tr("Re&cord"), waveBox);
     recordAction = new QAction(QIcon(seqrecord_xpm), tr("Re&cord"), waveBox);
@@ -307,8 +332,9 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
 
     QGridLayout *paramBoxLayout = new QGridLayout;
     paramBoxLayout->addWidget(loopBox, 0, 0, 1, 2);
-    paramBoxLayout->addWidget(recordButtonLabel, 1, 0);
-    paramBoxLayout->addWidget(recordButton, 1, 1);
+    paramBoxLayout->addWidget(muteOut, 1, 0, 1, 2);
+    paramBoxLayout->addWidget(recordButtonLabel, 2, 0);
+    paramBoxLayout->addWidget(recordButton, 2, 1);
     paramBoxLayout->addWidget(waveFormBoxLabel, 0, 2);
     paramBoxLayout->addWidget(waveFormBox, 0, 3);
     paramBoxLayout->addWidget(freqBoxLabel, 1, 2);
@@ -326,10 +352,11 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
 
     QGridLayout* waveBoxLayout = new QGridLayout;
     waveBoxLayout->addWidget(screen, 0, 0);
-    waveBoxLayout->addLayout(paramBoxLayout, 1, 0);
-    waveBoxLayout->addLayout(sliderLayout, 2, 0);
+    waveBoxLayout->addWidget(cursor, 1, 0);
+    waveBoxLayout->addLayout(paramBoxLayout, 2, 0);
+    waveBoxLayout->addLayout(sliderLayout, 3, 0);
     if (compactStyle) {
-        waveBoxLayout->setSpacing(1);
+        waveBoxLayout->setSpacing(0);
         waveBoxLayout->setMargin(2);
     }
     waveBox->setLayout(waveBoxLayout);
@@ -338,16 +365,20 @@ LfoWidget::LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
 
     QHBoxLayout *widgetLayout = new QHBoxLayout;
     widgetLayout->addWidget(waveBox, 1);
-    widgetLayout->addLayout(inOutBoxLayout, 0);
+    widgetLayout->addWidget(hideInOutBoxButton, 0);
+    widgetLayout->addWidget(inOutBox, 0);
 
     setLayout(widgetLayout);
     updateAmp(64);
 
     lastMute = false;
+    dataChanged = false;
+    needsGUIUpdate = false;
 }
 
 LfoWidget::~LfoWidget()
 {
+    delete parStore;
 }
 
 MidiLfo *LfoWidget::getMidiWorker()
@@ -362,6 +393,7 @@ void LfoWidget::writeData(QXmlStreamWriter& xml)
 
     xml.writeStartElement(manageBox->name.left(3));
     xml.writeAttribute("name", manageBox->name.mid(manageBox->name.indexOf(':') + 1));
+    xml.writeAttribute("inOutVisible", QString::number(inOutBox->isVisible()));
         xml.writeStartElement("input");
             xml.writeTextElement("enableNoteOff", QString::number(
                 midiWorker->enableNoteOff));
@@ -407,7 +439,7 @@ void LfoWidget::writeData(QXmlStreamWriter& xml)
 
         tempArray.clear();
         l1 = 0;
-        while (l1 < midiWorker->muteMask.count()) {
+        while (l1 < midiWorker->maxNPoints) {
             tempArray.append(midiWorker->muteMask.at(l1));
             l1++;
         }
@@ -417,7 +449,7 @@ void LfoWidget::writeData(QXmlStreamWriter& xml)
 
         tempArray.clear();
         l1 = 0;
-        while (l1 < midiWorker->muteMask.count()) {
+        while (l1 < midiWorker->maxNPoints) {
             tempArray.append(midiWorker->customWave.at(l1).value);
             l1++;
         }
@@ -530,12 +562,12 @@ void LfoWidget::readData(QXmlStreamReader& xml)
                 if (xml.isEndElement())
                     break;
                 if (xml.isStartElement() && (xml.name() == "data")) {
-                    midiWorker->muteMask.clear();
                     QByteArray tmpArray =
                             QByteArray::fromHex(xml.readElementText().toLatin1());
                     for (int l1 = 0; l1 < tmpArray.count(); l1++) {
-                        midiWorker->muteMask.append(tmpArray.at(l1));
+                        midiWorker->muteMask.replace(l1, tmpArray.at(l1));
                     }
+                    midiWorker->maxNPoints = tmpArray.count();
                 }
                 else skipXmlElement(xml);
             }
@@ -546,7 +578,6 @@ void LfoWidget::readData(QXmlStreamReader& xml)
                 if (xml.isEndElement())
                     break;
                 if (xml.isStartElement() && (xml.name() == "data")) {
-                    midiWorker->customWave.clear();
                     QByteArray tmpArray =
                             QByteArray::fromHex(xml.readElementText().toLatin1());
                     int step = TPQN / midiWorker->res;
@@ -555,7 +586,7 @@ void LfoWidget::readData(QXmlStreamReader& xml)
                         sample.value = tmpArray.at(l1);
                         sample.tick = lt;
                         sample.muted = midiWorker->muteMask.at(l1);
-                        midiWorker->customWave.append(sample);
+                        midiWorker->customWave.replace(l1, sample);
                         lt+=step;
                     }
                 }
@@ -631,42 +662,41 @@ void LfoWidget::updateTrigLegato(bool on)
 void LfoWidget::updateWaveForm(int val)
 {
     if (val > 5) return;
+    waveFormBoxIndex = val;
     midiWorker->updateWaveForm(val);
     midiWorker->getData(&data);
-    screen->updateScreen(data);
+    screen->updateData(data);
     bool isCustom = (val == 5);
     if (isCustom) newCustomOffset();
     amplitude->setDisabled(isCustom);
     freqBox->setDisabled(isCustom);
     modified = true;
+
 }
 
 void LfoWidget::updateScreen(int val)
 {
-    if (midiWorker->isRecording) {
-        midiWorker->getData(&data);
-        screen->updateScreen(data);
-    }
-    else {
-        screen->updateScreen(val);
-    }
+    if (!midiWorker->isRecording)
+        cursor->updatePosition(val);
 }
 
 void LfoWidget::updateFreq(int val)
 {
     if (val > 13) return;
+    freqBoxIndex = val;
     midiWorker->updateFrequency(lfoFreqValues[val]);
     midiWorker->getData(&data);
-    screen->updateScreen(data);
+    screen->updateData(data);
     modified = true;
 }
 
 void LfoWidget::updateRes(int val)
 {
     if (val > 8) return;
+    resBoxIndex = val;
     midiWorker->updateResolution(lfoResValues[val]);
     midiWorker->getData(&data);
-    screen->updateScreen(data);
+    screen->updateData(data);
     newCustomOffset();
     modified = true;
 }
@@ -674,9 +704,10 @@ void LfoWidget::updateRes(int val)
 void LfoWidget::updateSize(int val)
 {
     if (val > 11) return;
+    sizeBoxIndex = val;
     midiWorker->updateSize(sizeBox->currentText().toInt());
     midiWorker->getData(&data);
-    screen->updateScreen(data);
+    screen->updateData(data);
     newCustomOffset();
     modified = true;
 }
@@ -692,7 +723,7 @@ void LfoWidget::updateAmp(int val)
 {
     midiWorker->updateAmplitude(val);
     midiWorker->getData(&data);
-    screen->updateScreen(data);
+    screen->updateData(data);
     modified = true;
 }
 
@@ -700,7 +731,7 @@ void LfoWidget::updateOffs(int val)
 {
     midiWorker->updateOffset(val);
     midiWorker->getData(&data);
-    screen->updateScreen(data);
+    screen->updateData(data);
     modified = true;
 }
 
@@ -739,7 +770,7 @@ void LfoWidget::mouseMoved(double mouseX, double mouseY, int buttons)
         newCustomOffset();
     }
     midiWorker->getData(&data);
-    screen->updateScreen(data);
+    screen->updateData(data);
     modified = true;
 }
 
@@ -756,7 +787,7 @@ void LfoWidget::mousePressed(double mouseX, double mouseY, int buttons)
         newCustomOffset();
     }
     midiWorker->getData(&data);
-    screen->updateScreen(data);
+    screen->updateData(data);
     modified = true;
 }
 
@@ -768,6 +799,12 @@ void LfoWidget::mouseWheel(int step)
     offset->setValue(cv + step);
 }
 
+void LfoWidget::setInOutBoxVisible(bool on)
+{
+    inOutBox->setVisible(on);
+    modified=true;
+}
+
 void LfoWidget::setMuted(bool on)
 {
     midiWorker->setMuted(on);
@@ -836,8 +873,8 @@ void LfoWidget::storeParams(int ix, bool empty)
     parStore->temp.offs = offset->value();
     parStore->temp.waveForm = waveFormBox->currentIndex();
 
-    parStore->temp.wave = getCustomWave();
-    parStore->temp.muteMask = midiWorker->muteMask;
+    parStore->temp.wave = getCustomWave().mid(0, midiWorker->maxNPoints);
+    parStore->temp.muteMask = midiWorker->muteMask.mid(0, midiWorker->maxNPoints);
 
     parStore->tempToList(ix);
 }
@@ -845,8 +882,10 @@ void LfoWidget::storeParams(int ix, bool empty)
 void LfoWidget::restoreParams(int ix)
 {
     if (parStore->list.at(ix).empty) return;
-    midiWorker->customWave = parStore->list.at(ix).wave;
-    midiWorker->muteMask = parStore->list.at(ix).muteMask;
+    for (int l1 = 0; l1 < parStore->list.at(ix).wave.count(); l1++) {
+        midiWorker->customWave.replace(l1, parStore->list.at(ix).wave.at(l1));
+        midiWorker->muteMask.replace(l1, parStore->list.at(ix).muteMask.at(l1));
+    }
     sizeBox->setCurrentIndex(parStore->list.at(ix).size);
     midiWorker->updateSize(sizeBox->currentText().toInt());
 
@@ -858,7 +897,7 @@ void LfoWidget::restoreParams(int ix)
     waveFormBox->setCurrentIndex(parStore->list.at(ix).waveForm);
     loopBox->setCurrentIndex(parStore->list.at(ix).loopMode);
 
-    muteOut->setChecked(parStore->list.at(ix).muteOut);
+    //muteOut->setChecked(parStore->list.at(ix).muteOut);
     chIn->setCurrentIndex(parStore->list.at(ix).chIn);
     updateChIn(parStore->list.at(ix).chIn);
     ccnumberInBox->setValue(parStore->list.at(ix).ccnumberIn);
@@ -916,10 +955,9 @@ void LfoWidget::copyParamsFrom(LfoWidget *fromWidget)
     amplitude->setValue(fromWidget->amplitude->value());
     offset->setValue(fromWidget->offset->value());
 
-    midiWorker->customWave = fromWidget->getCustomWave();
-    midiWorker->muteMask.clear();
-    for (int l1 = 0; l1 < midiWorker->customWave.count(); l1++) {
-        midiWorker->muteMask.append(midiWorker->customWave.at(l1).muted);
+    for (int l1 = 0; l1 < fromWidget->getMidiWorker()->maxNPoints; l1++) {
+        midiWorker->customWave.replace(l1, fromWidget->getCustomWave().at(l1));
+        midiWorker->muteMask.replace(l1, midiWorker->customWave.at(l1).muted);
     }
     midiControl->setCcList(fromWidget->midiControl->ccList);
     muteOut->setChecked(true);
@@ -948,73 +986,118 @@ void LfoWidget::handleController(int ccnumber, int channel, int value)
             switch (cclist.at(l2).ID) {
                 case 0: if (min == max) {
                             if (value == max) {
-                                m = muteOut->isChecked();
-                                muteOut->setChecked(!m);
+                                m = midiWorker->isMuted;
+                                midiWorker->setMuted(!m);
                             }
                         }
                         else {
                             if (value == max) {
-                                muteOut->setChecked(false);
+                                midiWorker->setMuted(false);
                             }
                             if (value == min) {
-                                muteOut->setChecked(true);
+                                midiWorker->setMuted(true);
                             }
                         }
                 break;
 
                 case 1:
                         sval = min + ((double)value * (max - min) / 127);
-                        amplitude->setValue(sval);
+                        midiWorker->updateAmplitude(sval);
                 break;
 
                 case 2:
                         sval = min + ((double)value * (max - min) / 127);
-                        offset->setValue(sval);
+                        midiWorker->updateOffset(sval);
                 break;
                 case 3:
                         sval = min + ((double)value * (max - min) / 127);
-                        waveFormBox->setCurrentIndex(sval);
-                        updateWaveForm(sval);
+                        if (sval < 6) waveFormBoxIndex = sval;
                 break;
                 case 4:
                         sval = min + ((double)value * (max - min) / 127);
-                        freqBox->setCurrentIndex(sval);
-                        updateFreq(sval);
+                        if (sval < 14) freqBoxIndex = sval;
                 break;
                 case 5: if (min == max) {
                             if (value == max) {
-                                m = recordAction->isChecked();
-                                recordAction->setChecked(!m);
+                                m = midiWorker->recordMode;
+                                setRecord(!m);
+                                return;
                             }
                         }
                         else {
                             if (value == max) {
-                                recordAction->setChecked(true);
+                                setRecord(true);
                             }
                             if (value == min) {
-                                recordAction->setChecked(false);
+                                setRecord(false);
                             }
                         }
                 break;
                 case 6:
                         sval = min + ((double)value * (max - min) / 127);
-                        resBox->setCurrentIndex(sval);
-                        updateRes(sval);
+                        if (sval < 9) resBoxIndex = sval;
                 break;
                 case 7:
                         sval = min + ((double)value * (max - min) / 127);
-                        sizeBox->setCurrentIndex(sval);
-                        updateSize(sval);
+                        if (sval < 12) sizeBoxIndex = sval;
                 break;
                 case 8:
                         sval = min + ((double)value * (max - min) / 127);
-                        loopBox->setCurrentIndex(sval);
-                        updateLoop(sval);
+                        if (sval < 6) midiWorker->curLoopMode = sval;
                 break;
+                case 9:
+                        sval = min + ((double)value * (max - min) / 127);
+                        if ((sval < parStore->list.count())
+                                && (sval != parStore->activeStore)
+                                && (sval != parStore->currentRequest)) {
+                            parStore->requestDispState(sval, 2);
+                            parStore->restoreRequest = sval;
+                            parStore->restoreRunOnce = (parStore->jumpToList.at(sval) > -2);
+                        }
+                        else return;
+                break;
+
 
                 default:
                 break;
             }
+            needsGUIUpdate = true;
         }
     }
 }
+
+void LfoWidget::updateDisplay()
+{
+    QVector<Sample> data;
+
+    parStore->updateDisplay(getFramePtr()/midiWorker->frameSize, midiWorker->reverse);
+
+    if (midiWorker->dataChanged) {
+        midiWorker->getData(&data);
+        screen->updateData(data);
+        cursor->updateNumbers(midiWorker->res, midiWorker->size);
+        midiWorker->dataChanged = false;
+    }
+    screen->updateDraw();
+    cursor->updateDraw();
+    midiControl->update();
+
+    if (!needsGUIUpdate) return;
+
+    muteOut->setChecked(midiWorker->isMuted);
+    recordAction->setChecked(midiWorker->recordMode);
+    resBox->setCurrentIndex(resBoxIndex);
+    updateRes(resBoxIndex);
+    sizeBox->setCurrentIndex(sizeBoxIndex);
+    updateSize(sizeBoxIndex);
+    freqBox->setCurrentIndex(freqBoxIndex);
+    updateFreq(freqBoxIndex);
+    loopBox->setCurrentIndex(midiWorker->curLoopMode);
+    amplitude->setValue(midiWorker->amp);
+    offset->setValue(midiWorker->offs);
+    if (waveFormBoxIndex != waveFormBox->currentIndex()) {
+        waveFormBox->setCurrentIndex(waveFormBoxIndex);
+        updateWaveForm(waveFormBoxIndex);
+    }
+    needsGUIUpdate = false;
+}
