@@ -32,15 +32,18 @@
 #include <QString>
 #include <QTextStream>
 #include <QToolButton>
+#include <QPushButton>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
 
+#include "globstore.h"
 #include "midilfo.h"
 #include "slider.h"
 #include "lfoscreen.h"
 #include "midicontrol.h"
 #include "managebox.h"
 #include "parstore.h"
+#include "cursor.h"
 
 /*! This array holds the currently available frequency values.
  */
@@ -64,9 +67,12 @@ class LfoWidget : public QWidget
     Q_OBJECT
 
     MidiLfo *midiWorker;
+    GlobStore *globStore;
     QVector<Sample> data;
     bool modified;              /**< Is set to True if unsaved parameter modifications exist */
     bool lastMute;              /**< Contains the mute state of the last waveForm point modified by mouse click*/
+    bool dataChanged;
+    bool needsGUIUpdate;
 /*!
 * @brief This function populates the LfoWidget::waveForms list with
 * waveform names.
@@ -104,13 +110,16 @@ class LfoWidget : public QWidget
  * @param mutedAdd If set to True, the module will be added in muted state
  * @param parent The parent widget of this module, i.e. MainWindow
  */
-    LfoWidget(MidiLfo *p_midiWorker, int portCount, bool compactStyle,
-            bool mutedAdd = false, QWidget* parent=0);
+    LfoWidget(MidiLfo *p_midiWorker, GlobStore *p_globStore,
+            int portCount, bool compactStyle,
+            bool mutedAdd = false, bool inOutVisible = true,
+            const QString& name = "", QWidget* parent=0);
     ~LfoWidget();
 
     ParStore *parStore;
     MidiControl *midiControl;
     LfoScreen *screen;
+    Cursor *cursor;
     ManageBox *manageBox;
 
     QStringList waveForms;
@@ -120,17 +129,23 @@ class LfoWidget : public QWidget
     QComboBox *channelOut, *portOut;
     QComboBox *resBox, *sizeBox;
     QComboBox *loopBox;
-    QCheckBox *muteOut;
+    QWidget *inOutBox;
+    QPushButton *muteOut;
     QCheckBox *enableNoteOff;
     QCheckBox *enableRestartByKbd;
     QCheckBox *enableTrigByKbd;
     QCheckBox *enableTrigLegato;
     Slider *frequency, *amplitude, *offset;
     QAction *recordAction;
+    QAction* hideInOutBoxAction;
     QComboBox *waveFormBox, *freqBox;
 
     void setChIn(int value);
     QVector<Sample> getCustomWave();
+    int resBoxIndex;
+    int sizeBoxIndex;
+    int freqBoxIndex;
+    int waveFormBoxIndex;
 
 /*!
 * @brief This function returns the MidiLfo instance associated with this GUI
@@ -154,20 +169,6 @@ class LfoWidget : public QWidget
 */
     void writeData(QXmlStreamWriter& xml);
 /*!
-* @brief This function stores some module parameters in a parameter
-* list object
-*
-* @param Position index in the parameter list
-*/
-    void storeParams(int ix, bool empty = false);
-/*!
-* @brief This function restores some module parameters from the parameter
-* list object
-*
-* @param Position index in the parameter list
-*/
-    void restoreParams(int ix);
-/*!
 * @brief This function copies all LFO module GUI parameters from
 * fromWidget
 *
@@ -216,6 +217,9 @@ class LfoWidget : public QWidget
     void updateEnableRestartByKbd(bool on);
     void updateEnableTrigByKbd(bool on);
     void updateTrigLegato(bool on);
+    void setInOutBoxVisible(bool on);
+
+    void updateDisplay();
 
 /*!
 * @brief Slot for the LfoWidget::waveFormBox combobox setting the waveform
@@ -366,6 +370,20 @@ class LfoWidget : public QWidget
 *
 */
     void setMuted(bool on);
+/*!
+* @brief This function stores some module parameters in a parameter
+* list object
+*
+* @param Position index in the parameter list
+*/
+    void storeParams(int ix, bool empty = false);
+/*!
+* @brief This function restores some module parameters from the parameter
+* list object
+*
+* @param Position index in the parameter list
+*/
+    void restoreParams(int ix);
 
     int getFramePtr() { return midiWorker->getFramePtr(); }
     int getNextTick() { return midiWorker->nextTick; }
