@@ -64,6 +64,9 @@
 #include "pixmaps/filesave.xpm"
 #include "pixmaps/filesaveas.xpm"
 #include "pixmaps/filequit.xpm"
+#include "pixmaps/iopanelshow.xpm"
+#include "pixmaps/iopanelhide.xpm"
+#include "pixmaps/midicontrol.xpm"
 
 
 static const char FILEEXT[] = ".qmax";
@@ -72,6 +75,7 @@ int MainWindow::sigpipe[2];
 
 MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
 {
+    jackFailed = false;
     filename = "";
     lastDir = QDir::homePath();
     alsaMidi = p_alsamidi;
@@ -95,12 +99,18 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
     globStoreWindow->setVisible(true);
 
     engine = new Engine(globStore, grooveWidget, p_portCount, alsaMidi, this);
-    if (!alsaMidi) connect(engine->driver, SIGNAL(jsEvent(int)), this, SLOT(jsAction(int)));
-
-    connect(globStore, SIGNAL(globStore(int)), engine,
-            SLOT(globStore(int)));
-    connect(globStore, SIGNAL(requestRestore(int, int)), engine,
-            SLOT(requestRestore(int, int)));
+    if (alsaMidi) {
+        connect(engine->jackSync, SIGNAL(j_shutdown()), this, SLOT(jackShutdown()));
+    }
+    else {
+        connect(engine->driver, SIGNAL(jsEvent(int)), this, SLOT(jsAction(int)));
+        connect(engine->driver, SIGNAL(j_shutdown()), this, SLOT(jackShutdown()));
+        if (engine->driver->callJack(p_portCount)) jackFailed = true;
+    }
+    connect(globStore, SIGNAL(store(int)), engine,
+            SLOT(store(int)));
+    connect(globStore, SIGNAL(requestRestore(int)), engine,
+            SLOT(requestRestore(int)));
     connect(globStore, SIGNAL(updateGlobRestoreTimeModule(int)), engine,
             SLOT(updateGlobRestoreTimeModule(int)));
     connect(globStore, SIGNAL(removeParStores(int)), engine,
@@ -158,12 +168,14 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
     fileSaveAction->setShortcut(QKeySequence(QKeySequence::Save));
     fileSaveAction->setToolTip(tr("Save current arpeggiator file"));
     connect(fileSaveAction, SIGNAL(triggered()), this, SLOT(fileSave()));
+    fileSaveAction->setDisabled(true);
 
     fileSaveAsAction = new QAction(QIcon(filesaveas_xpm), tr("Save &as..."),
             this);
     fileSaveAsAction->setToolTip(
             tr("Save current arpeggiator file with new name"));
     connect(fileSaveAsAction, SIGNAL(triggered()), this, SLOT(fileSaveAs()));
+    fileSaveAsAction->setDisabled(true);
 
     fileQuitAction = new QAction(QIcon(filequit_xpm), tr("&Quit"), this);
     fileQuitAction->setShortcut(QKeySequence(tr("Ctrl+Q", "File|Quit")));
@@ -198,19 +210,21 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
     jackSyncAction->setCheckable(true);
     connect(jackSyncAction, SIGNAL(toggled(bool)), this,
             SLOT(jackSyncToggle(bool)));
-    if (alsaMidi) {
-        connect(engine->jackSync, SIGNAL(j_shutdown()), this,
-                SLOT(jackShutdown()));
-    }
-    else {
-        connect(engine->driver, SIGNAL(j_shutdown()), this,
-                SLOT(jackShutdown()));
-    }
     jackSyncAction->setChecked(false);
     jackSyncAction->setDisabled(true);
 
     updateTransportStatus(false);
 
+    showAllIOAction = new QAction(tr("&Show all IO panels"), this);
+    showAllIOAction->setIcon(QIcon(iopanelshow_xpm));
+    connect(showAllIOAction, SIGNAL(triggered()), this, SLOT(showIO()));
+    showAllIOAction->setDisabled(true);
+
+    hideAllIOAction = new QAction(tr("&Hide all IO panels"), this);
+    hideAllIOAction->setIcon(QIcon(iopanelhide_xpm));
+    connect(hideAllIOAction, SIGNAL(triggered()), this, SLOT(hideIO()));
+    hideAllIOAction->setDisabled(true);
+
     QAction* viewLogAction = logWindow->toggleViewAction();
     viewLogAction->setIcon(QIcon(eventlog_xpm));
     viewLogAction->setText(tr("&Event Log"));
@@ -253,12 +267,15 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
     connect(fileRecentlyOpenedFiles, SIGNAL(triggered(QAction*)), this,
         SLOT(recentFileActivated(QAction*)));
 
+    viewMenu->addAction(showAllIOAction);
+    viewMenu->addAction(hideAllIOAction);
     viewMenu->addAction(viewLogAction);
     viewMenu->addAction(viewGrooveAction);
-    viewMenu->addAction(viewSettingsAction);
     viewMenu->addAction(viewGlobAction);
-    viewMenu->addAction(tr("&MIDI Controllers..."),
-            this, SLOT(showMidiCCDialog()));
+    viewMenu->addAction(QIcon(midicontrol_xpm), tr("&MIDI Controllers..."),
+            this, SLOT(showMidiCCDialog()))
+            ->setShortcut(QKeySequence(tr("Ctrl+M", "View|MidiControllers")));
+    viewMenu->addAction(viewSettingsAction);
 
     arpMenu->addAction(addArpAction);
     arpMenu->addAction(addLfoAction);
@@ -281,6 +298,9 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
             SLOT(ftb_update_orientation(Qt::Orientation)));
 
     controlToolBar = new QToolBar(tr("&Control Toolbar"), this);
+    controlToolBar->addAction(showAllIOAction);
+    controlToolBar->addAction(hideAllIOAction);
+    controlToolBar->addSeparator();
     controlToolBar->addAction(viewLogAction);
     controlToolBar->addAction(viewGrooveAction);
     controlToolBar->addAction(viewGlobAction);
@@ -316,14 +336,17 @@ MainWindow::MainWindow(int p_portCount, bool p_alsamidi)
     if (checkRcFile())
         readRcFile();
 
+    passWidget->setModified(false);
+
     if (!installSignalHandlers())
         qWarning("%s", "Signal handlers not installed!");
 
-    show();
+    if (!jackFailed || alsaMidi) show();
 }
 
 MainWindow::~MainWindow()
 {
+    clear();
 }
 
 void MainWindow::updateWindowTitle()
@@ -388,21 +411,21 @@ void MainWindow::seqNew()
     }
 }
 
-void MainWindow::addArp(const QString& name, bool fromfile)
+void MainWindow::addArp(const QString& name, bool fromfile, bool inOutVisible)
 {
     int count, widgetID;
     MidiArp *midiWorker = new MidiArp();
-    ArpWidget *moduleWidget = new ArpWidget(midiWorker,
+    ArpWidget *moduleWidget = new ArpWidget(midiWorker, globStore,
             engine->getPortCount(), passWidget->compactStyle,
-            passWidget->mutedAdd, this);
+            passWidget->mutedAdd, inOutVisible, name, this);
     connect(moduleWidget, SIGNAL(presetsChanged(const QString&, const
                     QString&, int)),
             this, SLOT(updatePatternPresets(const QString&, const
                     QString&, int)));
     connect(moduleWidget->manageBox, SIGNAL(moduleRemove(int)),
             this, SLOT(removeArp(int)));
-    connect(moduleWidget->manageBox, SIGNAL(dockRename(const QString&, int)),
-            this, SLOT(renameDock(const QString&, int)));
+    connect(moduleWidget->manageBox, SIGNAL(dockRename(const QString&, int, int)),
+            engine, SLOT(renameDock(const QString&, int, int)));
     connect(moduleWidget->midiControl, SIGNAL(setMidiLearn(int, int, int)),
             engine, SLOT(setMidiLearn(int, int, int)));
 
@@ -427,35 +450,35 @@ void MainWindow::addArp(const QString& name, bool fromfile)
     moduleWidget->midiControl->parentDockID = count;
     moduleWidget->setProperty("widgetID", widgetID);
     appendDock(moduleWidget, name, count);
+    connect(moduleWidget->parStore->topButton, SIGNAL(pressed())
+            , engine->moduleWindow(count), SLOT(raise()));
 
     checkIfFirstModule();
 }
 
-void MainWindow::addLfo(const QString& p_name, bool fromfile, int clonefrom)
+void MainWindow::addLfo(const QString& p_name, bool fromfile, int clonefrom, bool inOutVisible)
 {
     int widgetID, count;
-    QString name;
 
     MidiLfo *midiWorker = new MidiLfo();
-    LfoWidget *moduleWidget = new LfoWidget(midiWorker,
+    LfoWidget *moduleWidget = new LfoWidget(midiWorker, globStore,
             engine->getPortCount(), passWidget->compactStyle,
-            passWidget->mutedAdd, this);
+            passWidget->mutedAdd, inOutVisible, p_name, this);
     connect(moduleWidget->manageBox, SIGNAL(moduleRemove(int)),
             this, SLOT(removeLfo(int)));
     connect(moduleWidget->manageBox, SIGNAL(moduleClone(int)), this, SLOT(cloneLfo(int)));
-    connect(moduleWidget->manageBox, SIGNAL(dockRename(const QString&, int)),
-            this, SLOT(renameDock(const QString&, int)));
+    connect(moduleWidget->manageBox, SIGNAL(dockRename(const QString&, int, int)),
+            engine, SLOT(renameDock(const QString&, int, int)));
     connect(moduleWidget->midiControl, SIGNAL(setMidiLearn(int, int, int)),
             engine, SLOT(setMidiLearn(int, int, int)));
 
     widgetID = engine->lfoWidgetCount();
     if (clonefrom >= 0) {
-        name = engine->lfoWidget(clonefrom)->manageBox->name + "_0";
         moduleWidget->copyParamsFrom(engine->lfoWidget(clonefrom));
     }
-    else name = p_name;
 
-    moduleWidget->manageBox->name = name;
+    //TODO: transfer these items to constructor
+    moduleWidget->manageBox->name = p_name;
     moduleWidget->manageBox->ID = widgetID;
     moduleWidget->midiControl->ID = widgetID;
 
@@ -478,35 +501,33 @@ void MainWindow::addLfo(const QString& p_name, bool fromfile, int clonefrom)
     moduleWidget->manageBox->parentDockID = count;
     moduleWidget->midiControl->parentDockID = count;
     moduleWidget->setProperty("widgetID", widgetID);
-    appendDock(moduleWidget, name, count);
+    appendDock(moduleWidget, p_name, count);
+    connect(moduleWidget->parStore->topButton, SIGNAL(pressed())
+            , engine->moduleWindow(count), SLOT(raise()));
 
     checkIfFirstModule();
 }
 
-void MainWindow::addSeq(const QString& p_name, bool fromfile, int clonefrom)
+void MainWindow::addSeq(const QString& p_name, bool fromfile, int clonefrom, bool inOutVisible)
 {
     int widgetID, count;
-    QString name;
-    
+
     MidiSeq *midiWorker = new MidiSeq();
-    SeqWidget *moduleWidget = new SeqWidget(midiWorker,
+    SeqWidget *moduleWidget = new SeqWidget(midiWorker, globStore,
             engine->getPortCount(), passWidget->compactStyle,
-            passWidget->mutedAdd, this);
+            passWidget->mutedAdd, inOutVisible, p_name, this);
     connect(moduleWidget->manageBox, SIGNAL(moduleRemove(int)), this, SLOT(removeSeq(int)));
     connect(moduleWidget->manageBox, SIGNAL(moduleClone(int)), this, SLOT(cloneSeq(int)));
-    connect(moduleWidget->manageBox, SIGNAL(dockRename(const QString&, int)),
-            this, SLOT(renameDock(const QString&, int)));
+    connect(moduleWidget->manageBox, SIGNAL(dockRename(const QString&, int, int)),
+            engine, SLOT(renameDock(const QString&, int, int)));
     connect(moduleWidget->midiControl, SIGNAL(setMidiLearn(int, int, int)),
             engine, SLOT(setMidiLearn(int, int, int)));
 
     widgetID = engine->seqWidgetCount();
     if (clonefrom >= 0) {
-        name = engine->seqWidget(clonefrom)->manageBox->name + "_0";
         moduleWidget->copyParamsFrom(engine->seqWidget(clonefrom));
     }
-    else name = p_name;
-    
-    moduleWidget->manageBox->name = name;
+    moduleWidget->manageBox->name = p_name;
     moduleWidget->manageBox->ID = widgetID;
     moduleWidget->midiControl->ID = widgetID;
 
@@ -516,7 +537,7 @@ void MainWindow::addSeq(const QString& p_name, bool fromfile, int clonefrom)
     if (!fromfile) for (int l1 = 0; l1 < (globStore->widgetList.count() - 1); l1++) {
         moduleWidget->storeParams(l1, true);
     }
-    
+
     if (clonefrom >= 0) {
         midiWorker->reverse = engine->seqWidget(clonefrom)->getReverse();
         midiWorker->setCurrentIndex(engine->seqWidget(clonefrom)->getCurrentIndex());
@@ -529,19 +550,24 @@ void MainWindow::addSeq(const QString& p_name, bool fromfile, int clonefrom)
     moduleWidget->manageBox->parentDockID = count;
     moduleWidget->midiControl->parentDockID = count;
     moduleWidget->setProperty("widgetID", widgetID);
-    appendDock(moduleWidget, name, count);
-
+    appendDock(moduleWidget, p_name, count);
+    connect(moduleWidget->parStore->topButton, SIGNAL(pressed())
+            , engine->moduleWindow(count), SLOT(raise()));
     checkIfFirstModule();
 }
 
 void MainWindow::cloneLfo(int ID)
 {
-    addLfo("", false, ID);
+    QString name;
+    name = engine->lfoWidget(ID)->manageBox->name + "_0";
+    addLfo(name, false, ID);
 }
 
 void MainWindow::cloneSeq(int ID)
 {
-    addSeq("", false, ID);
+    QString name;
+    name = engine->seqWidget(ID)->manageBox->name + "_0";
+    addSeq(name, false, ID);
 }
 
 void MainWindow::appendDock(QWidget *moduleWidget, const QString &name, int count)
@@ -557,13 +583,6 @@ void MainWindow::appendDock(QWidget *moduleWidget, const QString &name, int coun
     if (count) tabifyDockWidget(engine->moduleWindow(count - 1), moduleWindow);
     engine->addModuleWindow(moduleWindow);
     globStore->addModule(name);
-
-}
-
-void MainWindow::renameDock(const QString& name, int parentDockID)
-{
-    engine->moduleWindow(parentDockID)->setWindowTitle(name);
-    engine->setModified(true);
 }
 
 void MainWindow::removeArp(int index)
@@ -760,19 +779,24 @@ void MainWindow::readFilePartGlobal(QXmlStreamReader& xml)
             grooveWidget->readData(xml);
         else skipXmlElement(xml);
     }
+    passWidget->setModified(false);
 }
 
 void MainWindow::readFilePartModules(QXmlStreamReader& xml)
 {
     int count = 0;
+    bool iovis;
 
 
     while (!xml.atEnd()) {
+        iovis = true;
         xml.readNext();
         if (xml.isEndElement())
             break;
         if (xml.isStartElement() && (xml.name() == "Arp")) {
-            addArp("Arp:" + xml.attributes().value("name").toString(), true);
+            if (xml.attributes().hasAttribute("inOutVisible"))
+                iovis = xml.attributes().value("inOutVisible").toString().toInt();
+            addArp("Arp:" + xml.attributes().value("name").toString(), true, iovis);
             engine->arpWidget(engine->midiArpCount() - 1)
                     ->readData(xml);
             count++;
@@ -783,7 +807,9 @@ void MainWindow::readFilePartModules(QXmlStreamReader& xml)
             }
         }
         else if (xml.isStartElement() && (xml.name() == "LFO")) {
-            addLfo("LFO:" + xml.attributes().value("name").toString(), true);
+            if (xml.attributes().hasAttribute("inOutVisible"))
+                iovis = xml.attributes().value("inOutVisible").toString().toInt();
+            addLfo("LFO:" + xml.attributes().value("name").toString(), true, -1, iovis);
             engine->lfoWidget(engine->midiLfoCount() - 1)
                     ->readData(xml);
             count++;
@@ -794,7 +820,9 @@ void MainWindow::readFilePartModules(QXmlStreamReader& xml)
             }
         }
         else if (xml.isStartElement() && (xml.name() == "Seq")) {
-            addSeq("Seq:" + xml.attributes().value("name").toString(), true);
+            if (xml.attributes().hasAttribute("inOutVisible"))
+                iovis = xml.attributes().value("inOutVisible").toString().toInt();
+            addSeq("Seq:" + xml.attributes().value("name").toString(), true, -1, iovis);
             engine->seqWidget(engine->midiSeqCount() - 1)
                     ->readData(xml);
             count++;
@@ -979,13 +1007,14 @@ void MainWindow::closeEvent(QCloseEvent* e)
     if (isSave()) {
         writeRcFile();
         e->accept();
-    } else
+    }
+    else
         e->ignore();
 }
 
 bool MainWindow::isModified()
 {
-    return engine->isModified();
+    return (engine->isModified() || passWidget->isModified());
 }
 
 void MainWindow::updateTempo(int p_tempo)
@@ -1015,14 +1044,33 @@ void MainWindow::jackSyncToggle(bool on)
 {
     if (on) midiClockAction->setChecked(false);
     setGUIforExtSync(on);
-    engine->driver->setUseJackTransport(on);
+    engine->setUseJackTransport(on);
+}
+
+void MainWindow::showIO()
+{
+    engine->showAllIOPanels(true);
+}
+
+void MainWindow::hideIO()
+{
+    engine->showAllIOPanels(false);
+    resize(10, 10);
 }
 
 void MainWindow::jackShutdown()
 {
-    engine->setStatus(false);
-    jackSyncAction->setChecked(false);
-    jackSyncAction->setDisabled(true);
+    if (!alsaMidi) {
+        QMessageBox::warning(this, PACKAGE,
+                tr("JACK has shut down or could not be started, but you are trying\n"
+                   "to run QMidiArp with JACK MIDI backend.\n\n"
+                    "Alternatively you can use the ALSA MIDI backend \n"
+                    "by calling qmidiarp -a"));
+    }
+    else {
+        engine->setStatus(false);
+        jackSyncAction->setChecked(false);
+    }
 }
 
 void MainWindow::setGUIforExtSync(bool on)
@@ -1230,6 +1278,10 @@ void MainWindow::checkIfLastModule()
         midiClockAction->setChecked(false);
         jackSyncAction->setDisabled(true);
         jackSyncAction->setChecked(false);
+        fileSaveAction->setDisabled(true);
+        fileSaveAsAction->setDisabled(true);
+        showAllIOAction->setDisabled(true);
+        hideAllIOAction->setDisabled(true);
     }
 }
 
@@ -1238,6 +1290,10 @@ void MainWindow::checkIfFirstModule()
     if (engine->moduleWindowCount() == 1) {
         if (alsaMidi) midiClockAction->setEnabled(true);
         jackSyncAction->setEnabled(true);
+        fileSaveAction->setEnabled(true);
+        fileSaveAsAction->setEnabled(true);
+        showAllIOAction->setEnabled(true);
+        hideAllIOAction->setEnabled(true);
         runAction->setEnabled(!(midiClockAction->isChecked()
                                 || jackSyncAction->isChecked()));
     }
