@@ -110,7 +110,7 @@ MidiArp::MidiArp()
     patternIndex = 0;
     grooveIndex = 0;
     patternLen = 0;
-    octave = 0;
+    semitone = 0;
     newCurrent = false;
     newNext = false;
     currentNoteTick = 0;
@@ -134,12 +134,15 @@ MidiArp::MidiArp()
     attack_time = 0.0;
     release_time = 0.0;
     sustain = false;
-    sustainBuffer.clear();
+    sustainBuffer.resize(64);
+    sustainBufferCount = 0;
     latch_mode = false;
-    latchBuffer.clear();
-    latchTimer = new QTimer(this);
-    latchTimer->setSingleShot(true);
-    connect(latchTimer, SIGNAL(timeout()), this, SLOT(purgeLatchBuffer()));
+    latchBuffer.resize(64);
+    latchBufferCount = 0;
+    lastLatchTick = 0;
+
+    returnNote.resize(128);
+    returnVelocity.resize(128);
 }
 
 MidiArp::~MidiArp(){
@@ -219,17 +222,17 @@ bool MidiArp::handleEvent(MidiEvent inEv, int tick, int keep_rel)
             return(false);
         }
         if (sustain) {
-            sustainBuffer.append(inEv.data);
+            sustainBuffer.replace(sustainBufferCount, inEv.data);
+            sustainBufferCount++;
             return(false);
         }
 
         if (latch_mode) {
-            latchBuffer.append(inEv.data);
-            if (latchBuffer.count() == noteCount) {
-                latchTimer->stop();
-            }
-            else {
-                latchTimer->start(200);
+            latchBuffer.replace(latchBufferCount, inEv.data);
+            latchBufferCount++;
+            if (latchBufferCount != noteCount) {
+                if ((uint)tick > (uint)(lastLatchTick + 30) && (latchBufferCount > 1)) purgeLatchBuffer();
+                lastLatchTick = tick;
             }
             return(false);
         }
@@ -348,6 +351,7 @@ void MidiArp::getNote(int *tick, int note[], int velocity[], int *length)
     if (restartFlag) advancePatternIndex(true);
 
     if (!patternIndex) initLoop();
+    chordSemitone[0] = semitone;
     do {
         if (patternLen)
             c = (pattern.at(patternIndex));
@@ -359,6 +363,7 @@ void MidiArp::getNote(int *tick, int note[], int velocity[], int *length)
                 tmpIndex[chordIndex] = c.digitValue() + noteOfs;
                 if (chordMode) {
                     chordIndex++;
+                    chordSemitone[chordIndex] = semitone;
                 }
                 gotCC = false;
                 pause = (c == 'p');
@@ -376,14 +381,20 @@ void MidiArp::getNote(int *tick, int note[], int velocity[], int *length)
                         chordMode = false;
                         gotCC = false;
                         break;
+                    case 't':
+                        semitone++;
+                        break;
+                    case 'g':
+                        semitone--;
+                        break;
                     case '+':
-                        octave++;
+                        semitone+=12;
                         break;
                     case '-':
-                        octave--;
+                        semitone-=12;
                         break;
                     case '=':
-                        octave=0;
+                        semitone = 0;
                         break;
                     case '>':
                         stepWidth *= .5;
@@ -407,6 +418,7 @@ void MidiArp::getNote(int *tick, int note[], int velocity[], int *length)
                         len *= .5;
                         break;
                 }
+                chordSemitone[chordIndex] = semitone;
             }
         }
     } while (advancePatternIndex(false) && (gotCC || chordMode));
@@ -415,7 +427,7 @@ void MidiArp::getNote(int *tick, int note[], int velocity[], int *length)
     if (noteCount) do {
         noteIndex[l1] = (noteCount) ? tmpIndex[l1] % noteCount : 0;
         note[l1] = clip(notes[noteBufPtr][0][noteIndex[l1]]
-                + octave * 12, 0, 127, &outOfRange);
+                + chordSemitone[l1], 0, 127, &outOfRange);
         grooveTmp = (grooveIndex % 2) ? -grooveVelocity : grooveVelocity;
 
         if ((release_time > 0) && (notes[noteBufPtr][3][noteIndex[l1]])) {
@@ -451,8 +463,10 @@ void MidiArp::getNote(int *tick, int note[], int velocity[], int *length)
         else {
             l1++;
         }
-
-    } while ((tmpIndex[l1] >= 0) && (l1 < MAXCHORD - 1) && (l1 < noteCount));
+    } while (  (tmpIndex[l1] >= 0)
+            && (l1 < MAXCHORD - 1)
+            && ((l1 < noteCount) || (tmpIndex[l1] == 0))
+            && (noteCount));
 
     note[l1] = -1; // mark end of array
     grooveTmp = (grooveIndex % 2) ? -grooveLength : grooveLength;
@@ -519,7 +533,7 @@ void MidiArp::initLoop()
     stepWidth = 1.0;
     len = 0.5;
     vel = 0.8;
-    octave = 0;
+    semitone = 0;
     grooveIndex = 0;
 }
 
@@ -530,15 +544,12 @@ void MidiArp::prepareCurrentNote(int askedTick)
     int l1 = 0;
     updateNotes();
     returnTick = currentNoteTick;
-    returnNote.clear();
-    returnVelocity.clear();
-
     while ((currentNote[l1] >= 0) && (l1 < MAXCHORD - 1)) {
-        returnNote.append(currentNote[l1]);
-        returnVelocity.append(currentVelocity[l1]);
+        returnNote.replace(l1, currentNote[l1]);
+        returnVelocity.replace(l1, currentVelocity[l1]);
         l1++;
     }
-    returnNote.append(-1); // mark end of chord
+    returnNote.replace(l1, -1); // mark end of chord
     returnLength = currentLength;
     returnIsNew = newCurrent;
     newCurrent = false;
@@ -755,13 +766,13 @@ void MidiArp::updateTriggerMode(int val)
 void MidiArp::clearNoteBuffer()
 {
     noteCount = 0;
-    latchBuffer.clear();
+    latchBufferCount = 0;
     releaseNoteCount = 0;
 }
 
 int MidiArp::getPressedNoteCount()
 {
-    int c = noteCount - latchBuffer.count() - releaseNoteCount;
+    int c = noteCount - latchBufferCount - releaseNoteCount;
     return(c);
 }
 
@@ -776,11 +787,11 @@ void MidiArp::setSustain(bool on, int sustick)
 
 void MidiArp::purgeSustainBuffer(int sustick)
 {
-    for (int l1 = 0; l1 < sustainBuffer.count(); l1++) {
+    for (int l1 = 0; l1 < sustainBufferCount; l1++) {
         int buf = sustainBuffer.at(l1);
         removeNote(&buf, sustick, 1);
     }
-    sustainBuffer.clear();
+    sustainBufferCount = 0;
 }
 
 void MidiArp::setLatchMode(bool on)
@@ -791,11 +802,11 @@ void MidiArp::setLatchMode(bool on)
 
 void MidiArp::purgeLatchBuffer()
 {
-    for (int l1 = 0; l1 < latchBuffer.count(); l1++) {
+    for (int l1 = 0; l1 < latchBufferCount; l1++) {
         int buf = latchBuffer.at(l1);
         removeNote(&buf, arpTick, 1);
     }
-    latchBuffer.clear();
+    latchBufferCount = 0;
 }
 
 void MidiArp::newGrooveValues(int p_grooveTick, int p_grooveVelocity,
