@@ -29,7 +29,6 @@
 #include <QWidget>
 #include <QString>
 #include <QThread>
-#include <QTimer>
 #include <QVector>
 #include <main.h>
 
@@ -74,14 +73,15 @@ class MidiArp : public QWidget  {
     int grooveIndex; /*!< Holds the current position within the sequence*/
     int newGrooveTick, grooveTick, grooveVelocity, grooveLength;
     int randomTick, randomVelocity, randomLength;
+    int sustainBufferCount, latchBufferCount;
+    int lastLatchTick;
     double stepWidth, len, vel;
     QVector<int> sustainBuffer; /*!< Holds released note values when MidiArp::sustain is True */
     QVector<int> latchBuffer;   /*!< Holds released note values when MidiArp::latch_mode is True */
-    QTimer *latchTimer;         /*!< Is started when a note is released and causes MidiArp::purgeLatchBuffer after
-                                    a delay of 200ms except when another released note is detected before. */
 
     bool sustain, latch_mode;
-    int octave, noteIndex[MAXCHORD];
+    int noteIndex[MAXCHORD], chordSemitone[MAXCHORD];
+    int semitone;
  /*! @brief The input note buffer array of the Arpeggiator, which has
   * two array copies.
   *
