@@ -46,6 +46,7 @@ MidiLfo::MidiLfo()
     size = 1;
     res = 16;
     nPoints = 16;
+    maxNPoints = 16;
     old_res = 0;
     ccnumber = 74;
     portOut = 0;
@@ -64,26 +65,33 @@ MidiLfo::MidiLfo()
     int l1 = 0;
     int lt = 0;
     int step = TPQN / res;
+    cwmin = 0;
+
+    customWave.resize(8192);
     Sample sample;
     sample.value = 63;
-    customWave.clear();
-    cwmin = 0;
+    sample.tick = 0;
     for (l1 = 0; l1 < size * res; l1++) {
         sample.tick = lt;
         sample.muted = false;
-        customWave.append(sample);
+        customWave.replace(l1, sample);
         lt+=step;
     }
-    muteMask.fill(false, size * res);
+    muteMask.fill(false, 8192);
     data.clear();
+    frame.resize(32);
+    frame.fill(sample);
     lastMouseLoc = 0;
     lastMouseY = 0;
     frameptr = 0;
+    frameSize = 1;
     nextTick = 0;
     grooveTick = 0;
     newGrooveTick = 0;
     grooveVelocity = 0;
     grooveLength = 0;
+
+    dataChanged = false;
 }
 
 MidiLfo::~MidiLfo(){
@@ -94,27 +102,23 @@ void MidiLfo::setMuted(bool on)
     isMuted = on;
 }
 
-void MidiLfo::getNextFrame(QVector<Sample> *p_data, int tick)
+void MidiLfo::getNextFrame(int tick)
 {
     //this function is called by engine and returns one sample
     //if res <= LFO_FRAMELIMIT. If res > LFO_FRAMELIMIT, a frame is output
     //The FRAMELIMIT avoids excessive cursor updating
-
-    QVector<Sample> frame;
     Sample sample;
     const int step = TPQN / res;
     const int npoints = size * res;
     int lt, l1;
     int framelimit;
-    int framesize;
     int index;
 
-    frame.clear();
     gotKbdTrig = false;
 
     if (isRecording) framelimit = 32; else framelimit = LFO_FRAMELIMIT;
-    framesize = res / framelimit;
-    if (!framesize) framesize = 1;
+    frameSize = res / framelimit;
+    if (!frameSize) frameSize = 1;
 
     if (restartFlag) setFramePtr(0);
     if (!frameptr) grooveTick = newGrooveTick;
@@ -123,19 +127,20 @@ void MidiLfo::getNextFrame(QVector<Sample> *p_data, int tick)
     lt = nextTick;
     do {
         if (reverse) {
-            index = (framesize - 1 - l1 + frameptr) % npoints;
+            index = (frameSize - 1 - l1 + frameptr) % npoints;
         }
         else {
             index = (l1 + frameptr) % npoints;
         }
         sample = data.at(index);
+
         if (isRecording) {
-            if (framesize < 2) {
+            if (frameSize < 2) {
                 sample.value = recValue;
             }
             else {
             /* We do linear interpolation of points within frames if
-             * framesize is > 0 to get a smooth recording at high resolutions
+             * frameSize is > 0 to get a smooth recording at high resolutions
              * interpolation is linear between lastSampleValue and current recValue
              */
                 sample.value = lastSampleValue
@@ -143,13 +148,15 @@ void MidiLfo::getNextFrame(QVector<Sample> *p_data, int tick)
                             * ((double)l1 + .5);
             }
             customWave.replace(index, sample);
+            dataChanged = true;
         }
         sample.tick = lt;
         if (seqFinished) sample.muted = true;
-        frame.append(sample);
+        frame.replace(l1, sample);
         lt+=step;
         l1++;
-    } while ((l1 < framesize) & (l1 < npoints));
+    } while ((l1 < frameSize) & (l1 < npoints));
+
 
     reflect = pingpong;
 
@@ -176,7 +183,7 @@ void MidiLfo::getNextFrame(QVector<Sample> *p_data, int tick)
         }
     }
 
-    int cur_grv_sft = 0.01 * (grooveTick * step);
+    int cur_grv_sft = 0.01 * (grooveTick * (step - 1));
     /* pairwise application of new groove shift */
     if (!(frameptr % 2)) {
         cur_grv_sft = -cur_grv_sft;
@@ -190,17 +197,16 @@ void MidiLfo::getNextFrame(QVector<Sample> *p_data, int tick)
     if (nextTick < (tick - lt)) nextTick = tick;
     sample.value = -1;
     sample.tick = nextTick;
-    frame.append(sample);
+    frame.replace(l1, sample);
 
     if (!trigByKbd && !(frameptr % 2) && !grooveTick) {
         /* round-up to current resolution (quantize) */
-        nextTick/= (step * framesize);
-        nextTick*= (step * framesize);
+        nextTick/= (step * frameSize);
+        nextTick*= (step * frameSize);
     }
 
     if (seqFinished) frameptr = 0;
 
-    *p_data = frame;
 }
 
 void MidiLfo::getData(QVector<Sample> *p_data)
@@ -391,36 +397,32 @@ void MidiLfo::resizeAll()
 {
     int lt = 0;
     int l1 = 0;
-    int os;
     const int step = TPQN / res;
     const int npoints = res * size;
     Sample sample;
 
     frameptr%=npoints;
 
-    os = customWave.count();
-    if (os < npoints) {
-        customWave.resize(npoints);
-        muteMask.resize(npoints);
+    if (maxNPoints < npoints) {
         for (l1 = 0; l1 < npoints; l1++) {
-            if (l1 >= os) muteMask.replace(l1, muteMask.at(l1 % os));
-            sample = customWave.at(l1 % os);
+            if (l1 >= maxNPoints)
+                muteMask.replace(l1, muteMask.at(l1 % maxNPoints));
+            sample = customWave.at(l1 % maxNPoints);
             sample.tick = lt;
             sample.muted = muteMask.at(l1);
             customWave.replace(l1, sample);
             lt+=step;
         }
+        maxNPoints = npoints;
     }
     nPoints = npoints;
+    dataChanged = true;
 }
 
 void MidiLfo::copyToCustom()
 {
-    int m;
-
-    m = data.count();
-    data.remove(m - 1);
-    customWave = data;
+    for (int l1 = 0; l1 < nPoints; l1++)
+    customWave.replace(l1, data.at(l1));
 }
 
 void MidiLfo::updateCustomWaveOffset(int cwoffs)
