@@ -58,10 +58,10 @@ MidiSeq::MidiSeq()
     size = 4;
     res = 4;
     nPoints = 16;
+    maxNPoints = 16;
     notelength = 74;
     portOut = 0;
     channelOut = 0;
-    waveFormIndex = 0;
     currentIndex = 0;
     nextTick = 0;
     grooveTick = 0;
@@ -69,19 +69,22 @@ MidiSeq::MidiSeq()
     grooveVelocity = 0;
     grooveLength = 0;
     isMuted = false;
+    dataChanged = false;
+
     int lt = 0;
     int l1 = 0;
     int step = TPQN / res;
     Sample sample;
     sample.value = 60;
-    customWave.clear();
-    for (l1 = 0; l1 < size * res; l1++) {
+    customWave.resize(512);
+    muteMask.resize(512);
+    for (l1 = 0; l1 < 512; l1++) {
             sample.tick = lt;
             sample.muted = false;
-            customWave.append(sample);
+            customWave.replace(l1, sample);
             lt+=step;
     }
-    muteMask.fill(false, size * res);
+    muteMask.fill(false, 512);
 }
 
 MidiSeq::~MidiSeq(){
@@ -142,7 +145,7 @@ void MidiSeq::getNextNote(Sample *p_sample, int tick)
     sample.tick = nextTick;
 
 
-    cur_grv_sft = 0.01 * (grooveTick * frame_nticks);
+    cur_grv_sft = 0.01 * (grooveTick * (frame_nticks - 1));
 
     /* pairwise application of new groove shift */
     if (!(currentIndex % 2)) {
@@ -224,16 +227,9 @@ void MidiSeq::getData(QVector<Sample> *p_data)
     const int npoints = res * size;
 
     QVector<Sample> data;
-    data.clear();
 
-    switch(waveFormIndex) {
-        case 0: //custom
-            lt = step * npoints;
-            data = customWave.mid(0, npoints);
-        break;
-        default:
-        break;
-    }
+    lt = step * npoints;
+    data = customWave.mid(0, npoints);
     sample.value = -1;
     sample.tick = lt;
     data.append(sample);
@@ -255,11 +251,6 @@ int MidiSeq::clip(int value, int min, int max, bool *outOfRange)
     return(tmp);
 }
 
-void MidiSeq::updateWaveForm(int val)
-{
-    waveFormIndex = val;
-}
-
 void MidiSeq::updateLoop(int val)
 {
     backward = val&1;
@@ -331,7 +322,6 @@ void MidiSeq::resizeAll()
 {
     int lt = 0;
     int l1 = 0;
-    int os;
     const int step = TPQN / res;
     const int npoints = res * size;
     Sample sample;
@@ -339,33 +329,21 @@ void MidiSeq::resizeAll()
     currentIndex%=npoints;
     currentRecStep%=npoints;
 
-    os = customWave.count();
-    if (os < npoints) {
-        customWave.resize(npoints);
-        muteMask.resize(npoints);
+    if (maxNPoints < npoints) {
         for (l1 = 0; l1 < npoints; l1++) {
-            if (l1 >= os) muteMask.replace(l1, muteMask.at(l1 % os));
-            sample = customWave.at(l1 % os);
+            if (l1 >= maxNPoints)
+                muteMask.replace(l1, muteMask.at(l1 % maxNPoints));
+            sample = customWave.at(l1 % maxNPoints);
             sample.tick = lt;
             sample.muted = muteMask.at(l1);
             customWave.replace(l1, sample);
             lt+=step;
         }
+        maxNPoints = npoints;
     }
 
     if (!loopMarker) nPoints = npoints;
-}
-
-void MidiSeq::copyToCustom()
-{
-    QVector<Sample> data;
-    int m;
-
-    data.clear();
-    getData(&data);
-    m = data.count();
-    data.remove(m - 1);
-    customWave = data;
+    dataChanged = true;
 }
 
 bool MidiSeq::toggleMutePoint(double mouseX)
