@@ -70,7 +70,6 @@ class MidiSeq : public QObject  {
     bool backward;       /*!< True when the sequence should start backward */
     bool pingpong;      /*!< True when the play direction should alternate */
     bool reflect;      /*!< True when the current play direction will change at the next reflect point */
-    int curLoopMode;    /*!< Local storage of the currently active Loop mode */
     int noteCount;
     bool recordMode;
     bool seqFinished;
@@ -108,11 +107,14 @@ class MidiSeq : public QObject  {
     int portOut;        /**< Output port index */
     int channelOut;
     bool isMuted;
+    bool dataChanged;
     int vel, transp, notelength;
-    int size, res, waveFormIndex;
+    int size, res;
     int currentRecStep;
+    int curLoopMode;    /*!< Local storage of the currently active Loop mode */
     int loopMarker;
     int nPoints;        /*!< Number of steps to be played out */
+    int maxNPoints;        /*!< Maximum number of steps that have been used in the session */
     int nextTick;
     int nOctaves;
     int baseOctave;
@@ -211,10 +213,6 @@ class MidiSeq : public QObject  {
  * lead to a bigger size data array.
  */
     void resizeAll();
-/*! @brief Currently not in use. This function will copy the current
- * MidiSeq::data array into MidiSeq::customWave.
- */
-    void copyToCustom();
     void setRecordMode(int on);
     void setRecordedNote(int note);
 
