@@ -75,9 +75,11 @@ class SeqDriver : public DriverBase {
         int deltaToTick (double curtime);
         double aTimeToDelta(snd_seq_real_time_t* atime);
         const snd_seq_real_time_t* deltaToATime(double curtime);
+        snd_seq_remove_events_t *remove_ev;
         void calcClockRatio();
 
         void initTempo();
+        bool callJack(int portcount);
 
         JackDriver *jackSync;
         jack_position_t jPos;
