@@ -47,9 +47,7 @@ SeqScreen::SeqScreen(QWidget* parent) : QWidget (parent)
     grooveVelocity = 0;
     grooveLength = 0;
     isMuted = false;
-    timer = new QTimer(this);
-    timer->setSingleShot(true);
-    connect(timer, SIGNAL(timeout()), this, SLOT(update()));
+    needsRedraw = false;
 }
 
 SeqScreen::~SeqScreen()
@@ -98,7 +96,6 @@ void SeqScreen::paintEvent(QPaintEvent*)
     p.setViewport(0, 0, w, h);
     p.setWindow(0, 0, w, h);
     p.setPen(QColor(20, 20, 160));
-    p.drawRect(0, 0, w - 1, h - 1);
 
     //Draw current record step
     if (recordMode)
@@ -200,15 +197,6 @@ void SeqScreen::paintEvent(QPaintEvent*)
     p.setPen(pen);
     p.drawLine(SEQSCR_HMARG / 2, ypos, SEQSCR_HMARG *2 / 3, ypos);
 
-    // Cursor
-    pen.setWidth(notestreak_thick);
-    pen.setColor(QColor(50, 180, 220));
-    p.setPen(pen);
-    x = (currentIndex + .01 * (double)grooveTick * (currentIndex % 2))
-        * xscale * (int)nsteps / npoints;
-    xpos = SEQSCR_HMARG + x + pen.width() / 2;
-    p.drawLine(xpos, h - 2,
-                    xpos + (xscale / beatRes) - pen.width(), h - 2);
     // Loop Marker
     if (loopMarker) {
         QPolygon trg;
@@ -229,16 +217,10 @@ void SeqScreen::paintEvent(QPaintEvent*)
     }
 }
 
-void SeqScreen::updateScreen(const QVector<Sample>& data)
+void SeqScreen::updateData(const QVector<Sample>& data)
 {
     p_data = data;
-    update();
-}
-
-void SeqScreen::updateScreen(int p_index)
-{
-    currentIndex = p_index;
-    update();
+    needsRedraw = true;
 }
 
 void SeqScreen::newGrooveValues(int tick, int vel, int length)
@@ -246,13 +228,13 @@ void SeqScreen::newGrooveValues(int tick, int vel, int length)
     grooveTick = tick;
     grooveVelocity = vel;
     grooveLength = length;
-    if (!timer->isActive()) timer->start(50);
+    needsRedraw = true;
 }
 
 void SeqScreen::setMuted(bool on)
 {
     isMuted = on;
-    update();
+    needsRedraw = true;
 }
 
 void SeqScreen::mouseMoveEvent(QMouseEvent *event)
@@ -283,16 +265,25 @@ void SeqScreen::mousePressEvent(QMouseEvent *event)
 void SeqScreen::setRecordMode(bool on)
 {
     recordMode = on;
+    needsRedraw = true;
 }
 
 void SeqScreen::setCurrentRecStep(int recStep)
 {
     currentRecStep = recStep;
+    needsRedraw = true;
 }
 
 void SeqScreen::setLoopMarker(int pos)
 {
     loopMarker = pos;
+    needsRedraw = true;
+}
+
+void SeqScreen::updateDraw()
+{
+    if (!needsRedraw) return;
+    needsRedraw = false;
     update();
 }
 
