@@ -34,16 +34,23 @@
 #include "pixmaps/seqrecord.xpm"
 
 
-SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
-    bool mutedAdd, QWidget *parent):
-    QWidget(parent), midiWorker(p_midiWorker), modified(false)
+SeqWidget::SeqWidget(MidiSeq *p_midiWorker, GlobStore *p_globStore,
+    int portCount, bool compactStyle,
+    bool mutedAdd, bool inOutVisible, const QString& name, QWidget *parent):
+    QWidget(parent), midiWorker(p_midiWorker),
+    globStore(p_globStore), modified(false)
 {
     int l1;
 
-    parStore = new ParStore();
-
     midiControl = new MidiControl(this);
 
+    parStore = new ParStore(globStore, name, this);
+    midiControl->addMidiLearnMenu("Restore_"+name, parStore->topButton, 7);
+    connect(parStore, SIGNAL(store(int, bool)),
+             this, SLOT(storeParams(int, bool)));
+    connect(parStore, SIGNAL(restore(int)),
+             this, SLOT(restoreParams(int)));
+
     manageBox = new ManageBox("Seq:", true, this);
 
     // Display group box on right
@@ -161,12 +168,6 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     // Output group box on right bottom
     QGroupBox *portBox = new QGroupBox(tr("Output"), this);
 
-    QLabel *muteLabel = new QLabel(tr("&Mute"),portBox);
-    muteOut = new QCheckBox(this);
-    connect(muteOut, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
-    muteLabel->setBuddy(muteOut);
-    midiControl->addMidiLearnMenu("MuteToggle", muteOut, 0);
-
     QLabel *portLabel = new QLabel(tr("&Port"), portBox);
     portOut = new QComboBox(portBox);
     portLabel->setBuddy(portOut);
@@ -182,12 +183,10 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
 
 
     QGridLayout *portBoxLayout = new QGridLayout;
-    portBoxLayout->addWidget(muteLabel, 0, 0);
-    portBoxLayout->addWidget(muteOut, 0, 1);
-    portBoxLayout->addWidget(portLabel, 1, 0);
-    portBoxLayout->addWidget(portOut, 1, 1);
-    portBoxLayout->addWidget(channelLabel, 2, 0);
-    portBoxLayout->addWidget(channelOut, 2, 1);
+    portBoxLayout->addWidget(portLabel, 0, 0);
+    portBoxLayout->addWidget(portOut, 0, 1);
+    portBoxLayout->addWidget(channelLabel, 1, 0);
+    portBoxLayout->addWidget(channelOut, 1, 1);
 
     QVBoxLayout* outputLayout = new QVBoxLayout;
     outputLayout->addLayout(portBoxLayout);
@@ -198,12 +197,24 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
 
     portBox->setLayout(outputLayout);
 
+    hideInOutBoxAction = new QAction(tr("&Show/hide in-out settings"), this);
+    QToolButton *hideInOutBoxButton = new QToolButton(this);
+    hideInOutBoxAction->setCheckable(true);
+    hideInOutBoxAction->setChecked(inOutVisible);
+    hideInOutBoxButton->setDefaultAction(hideInOutBoxAction);
+    hideInOutBoxButton->setFixedSize(10, 80);
+    hideInOutBoxButton->setArrowType (Qt::ArrowType(0));
+    connect(hideInOutBoxAction, SIGNAL(toggled(bool)), this, SLOT(setInOutBoxVisible(bool)));
+
     QVBoxLayout *inOutBoxLayout = new QVBoxLayout;
     inOutBoxLayout->addWidget(manageBox);
     inOutBoxLayout->addWidget(dispBox);
     inOutBoxLayout->addWidget(inBox);
     inOutBoxLayout->addWidget(portBox);
     inOutBoxLayout->addStretch();
+    inOutBox = new QWidget(this);
+    inOutBox->setLayout(inOutBoxLayout);
+    inOutBox->setVisible(inOutVisible);
 
     // group box for sequence setup
     QGroupBox *seqBox = new QGroupBox(tr("Sequence"), this);
@@ -217,16 +228,14 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     connect(screen, SIGNAL(mousePressed(double, double, int)), this,
             SLOT(mousePressed(double, double, int)));
 
-    QLabel *waveFormBoxLabel = new QLabel(tr("&Sequence"), seqBox);
-    waveFormBox = new QComboBox(seqBox);
-    waveFormBoxLabel->setBuddy(waveFormBox);
-    loadWaveForms();
-    waveFormBox->insertItems(0, waveForms);
-    waveFormBox->setCurrentIndex(0);
-    waveFormBox->setToolTip(tr("Preset Number"));
-    waveFormBox->setMinimumContentsLength(8);
-    connect(waveFormBox, SIGNAL(activated(int)), this,
-            SLOT(updateWaveForm(int)));
+    cursor = new Cursor('S', this);
+
+    muteOut = new QPushButton(tr("&Mute"),this);
+    muteOut->setFont(QFont("Helvetica", 8));
+    muteOut->setMinimumSize(QSize(35,10));
+    muteOut->setCheckable(true);
+    connect(muteOut, SIGNAL(toggled(bool)), this, SLOT(setMuted(bool)));
+    midiControl->addMidiLearnMenu("MuteToggle", muteOut, 0);
 
     loopBox = new QComboBox(seqBox);
     QStringList names;
@@ -258,6 +267,7 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     names << "1" << "2" << "4" << "8" << "16";
     resBox->insertItems(0, names);
     resBox->setCurrentIndex(2);
+    resBoxIndex = 2;
     resBox->setToolTip(
             tr("Resolution (notes/beat): Number of notes produced every beat"));
     resBox->setMinimumContentsLength(3);
@@ -272,17 +282,13 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
     names << "1" << "2" << "3" << "4" << "5" << "6" << "7" << "8";
     sizeBox->insertItems(0, names);
     sizeBox->setCurrentIndex(3);
+    sizeBoxIndex = 3;
     sizeBox->setToolTip(tr("Length of Sequence in beats"));
     sizeBox->setMinimumContentsLength(3);
     connect(sizeBox, SIGNAL(activated(int)), this,
             SLOT(updateSize(int)));
     midiControl->addMidiLearnMenu("Size", sizeBox, 5);
 
-    //temporarily hide these elements until multiple patterns are implemented
-    waveFormBox->setEnabled(false);
-    waveFormBox->setVisible(false);
-    waveFormBoxLabel->setVisible(false);
-
 
     velocity = new Slider(0, 127, 1, 8, 64, Qt::Horizontal,
             tr("Veloc&ity"), seqBox);
@@ -315,8 +321,7 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
 
     QGridLayout *paramBoxLayout = new QGridLayout;
     paramBoxLayout->addWidget(loopBox, 0, 0, 1, 2);
-    paramBoxLayout->addWidget(waveFormBoxLabel, 1, 0);
-    paramBoxLayout->addWidget(waveFormBox, 1, 1);
+    paramBoxLayout->addWidget(muteOut, 1, 0, 1, 2);
     paramBoxLayout->addWidget(recordButtonLabel, 2, 0);
     paramBoxLayout->addWidget(recordButton, 2, 1);
     paramBoxLayout->addWidget(resBoxLabel, 3, 0);
@@ -327,11 +332,12 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
 
     QGridLayout* seqBoxLayout = new QGridLayout;
     seqBoxLayout->addWidget(screen, 0, 0, 1, 2);
-    seqBoxLayout->addLayout(paramBoxLayout, 1, 0);
-    seqBoxLayout->addLayout(sliderLayout, 1, 1);
+    seqBoxLayout->addWidget(cursor, 1, 0, 1, 2);
+    seqBoxLayout->addLayout(paramBoxLayout, 2, 0);
+    seqBoxLayout->addLayout(sliderLayout, 2, 1);
     if (compactStyle) {
         seqBoxLayout->setMargin(2);
-        seqBoxLayout->setSpacing(1);
+        seqBoxLayout->setSpacing(0);
     }
     seqBox->setLayout(seqBoxLayout);
 
@@ -339,10 +345,13 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
 
     QHBoxLayout *widgetLayout = new QHBoxLayout;
     widgetLayout->addWidget(seqBox, 1);
-    widgetLayout->addLayout(inOutBoxLayout, 0);
+    widgetLayout->addWidget(hideInOutBoxButton, 0);
+    widgetLayout->addWidget(inOutBox, 0);
 
     setLayout(widgetLayout);
     recordMode = false;
+    dataChanged=false;
+    needsGUIUpdate=false;
     updateVelocity(64);
     updateWaveForm(0);
     lastMute = false;
@@ -350,6 +359,7 @@ SeqWidget::SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
 
 SeqWidget::~SeqWidget()
 {
+    delete parStore;
 }
 
 MidiSeq *SeqWidget::getMidiWorker()
@@ -364,6 +374,7 @@ void SeqWidget::writeData(QXmlStreamWriter& xml)
 
     xml.writeStartElement(manageBox->name.left(3));
     xml.writeAttribute("name", manageBox->name.mid(manageBox->name.indexOf(':') + 1));
+    xml.writeAttribute("inOutVisible", QString::number(inOutBox->isVisible()));
 
         xml.writeStartElement("display");
             xml.writeTextElement("vertical", QString::number(
@@ -413,7 +424,7 @@ void SeqWidget::writeData(QXmlStreamWriter& xml)
 
         tempArray.clear();
         l1 = 0;
-        while (l1 < midiWorker->muteMask.count()) {
+        while (l1 < midiWorker->maxNPoints) {
             tempArray.append(midiWorker->muteMask.at(l1));
             l1++;
         }
@@ -423,7 +434,7 @@ void SeqWidget::writeData(QXmlStreamWriter& xml)
 
         tempArray.clear();
         l1 = 0;
-        while (l1 < midiWorker->muteMask.count()) {
+        while (l1 < midiWorker->maxNPoints) {
             tempArray.append(midiWorker->customWave.at(l1).value);
             l1++;
         }
@@ -443,7 +454,6 @@ void SeqWidget::writeData(QXmlStreamWriter& xml)
 void SeqWidget::readData(QXmlStreamReader& xml)
 {
     int tmp;
-    int wvtmp = 0;
     Sample sample;
 
     while (!xml.atEnd()) {
@@ -550,12 +560,12 @@ void SeqWidget::readData(QXmlStreamReader& xml)
                 if (xml.isEndElement())
                     break;
                 if (xml.isStartElement() && (xml.name() == "data")) {
-                    midiWorker->muteMask.clear();
                     QByteArray tmpArray =
                             QByteArray::fromHex(xml.readElementText().toLatin1());
                     for (int l1 = 0; l1 < tmpArray.count(); l1++) {
-                        midiWorker->muteMask.append(tmpArray.at(l1));
+                        midiWorker->muteMask.replace(l1, tmpArray.at(l1));
                     }
+                    midiWorker->maxNPoints = tmpArray.count();
                 }
                 else skipXmlElement(xml);
             }
@@ -566,7 +576,6 @@ void SeqWidget::readData(QXmlStreamReader& xml)
                 if (xml.isEndElement())
                     break;
                 if (xml.isStartElement() && (xml.name() == "data")) {
-                    midiWorker->customWave.clear();
                     QByteArray tmpArray =
                             QByteArray::fromHex(xml.readElementText().toLatin1());
                     int step = TPQN / midiWorker->res;
@@ -575,9 +584,10 @@ void SeqWidget::readData(QXmlStreamReader& xml)
                         sample.value = tmpArray.at(l1);
                         sample.tick = lt;
                         sample.muted = midiWorker->muteMask.at(l1);
-                        midiWorker->customWave.append(sample);
+                        midiWorker->customWave.replace(l1, sample);
                         lt+=step;
                     }
+                    updateWaveForm(0);
                 }
                 else if (xml.name() == "loopmarker") {
                     tmp = xml.readElementText().toInt();
@@ -595,16 +605,9 @@ void SeqWidget::readData(QXmlStreamReader& xml)
         }
         else skipXmlElement(xml);
     }
-    waveFormBox->setCurrentIndex(wvtmp);
-    updateWaveForm(wvtmp);
     modified = false;
 }
 
-void SeqWidget::loadWaveForms()
-{
-    waveForms << tr("Custom");
-}
-
 void SeqWidget::setEnableNoteIn(bool on)
 {
     enableNoteIn->setChecked(on);
@@ -673,9 +676,9 @@ void SeqWidget::updateNoteLength(int val)
 
 void SeqWidget::updateWaveForm(int val)
 {
-    midiWorker->updateWaveForm(val);
+    (void)val;
     midiWorker->getData(&data);
-    screen->updateScreen(data);
+    screen->updateData(data);
     modified = true;
 }
 
@@ -685,28 +688,29 @@ void SeqWidget::setRecord(bool on)
     midiWorker->setRecordMode(on);
     screen->setRecordMode(on);
     screen->setCurrentRecStep(midiWorker->currentRecStep);
-    screen->update();
 }
 
 void SeqWidget::updateRes(int val)
 {
     if (val > 4) return;
+    resBoxIndex = val;
     midiWorker->res = seqResValues[val];
     midiWorker->resizeAll();
     midiWorker->getData(&data);
     screen->setCurrentRecStep(midiWorker->currentRecStep);
-    screen->updateScreen(data);
+    screen->updateData(data);
     modified = true;
 }
 
 void SeqWidget::updateSize(int val)
 {
     if (val > 7) return;
+    sizeBoxIndex = val;
     midiWorker->size = val + 1;
     midiWorker->resizeAll();
     midiWorker->getData(&data);
     screen->setCurrentRecStep(midiWorker->currentRecStep);
-    screen->updateScreen(data);
+    screen->updateData(data);
     modified = true;
 }
 
@@ -733,24 +737,16 @@ void SeqWidget::updateTranspose(int val)
 void SeqWidget::processNote(int note, int vel)
 {
     if (!recordMode) {
-        if (enableNoteIn->isChecked()) transpose->setValue(note - 60);
-        if (enableVelIn->isChecked()) velocity->setValue(vel);
+        if (enableNoteIn->isChecked()) updateTranspose(note - 60);
+        if (enableVelIn->isChecked()) updateVelocity(vel);
+        needsGUIUpdate = true;
     }
     else {
-        midiWorker->getData(&data);
         screen->setCurrentRecStep(midiWorker->currentRecStep);
-        screen->updateScreen(data);
+        dataChanged=true;
     }
 }
 
-void SeqWidget::copyToCustom()
-{
-    midiWorker->copyToCustom();
-    waveFormBox->setCurrentIndex(0);
-    updateWaveForm(0);
-    modified = true;
-}
-
 void SeqWidget::mouseMoved(double mouseX, double mouseY, int buttons)
 {
     if ((mouseX > 1) || (mouseX < 0) || (mouseY > 1)) return;
@@ -762,7 +758,7 @@ void SeqWidget::mouseMoved(double mouseX, double mouseY, int buttons)
         screen->setCurrentRecStep(midiWorker->currentRecStep);
     }
     midiWorker->getData(&data);
-    screen->updateScreen(data);
+    screen->updateData(data);
     modified = true;
 }
 
@@ -786,10 +782,16 @@ void SeqWidget::mousePressed(double mouseX, double mouseY, int buttons)
         screen->setCurrentRecStep(midiWorker->currentRecStep);
     }
     midiWorker->getData(&data);
-    screen->updateScreen(data);
+    screen->updateData(data);
     modified = true;
 }
 
+void SeqWidget::setInOutBoxVisible(bool on)
+{
+    inOutBox->setVisible(on);
+    modified=true;
+}
+
 void SeqWidget::setMuted(bool on)
 {
     midiWorker->setMuted(on);
@@ -886,8 +888,8 @@ void SeqWidget::storeParams(int ix, bool empty)
     parStore->temp.vel = velocity->value();
     parStore->temp.dispVertical = dispVertical;
     parStore->temp.loopMode = loopBox->currentIndex();
-    parStore->temp.wave = getCustomWave();
-    parStore->temp.muteMask = midiWorker->muteMask;
+    parStore->temp.wave = getCustomWave().mid(0, midiWorker->maxNPoints);
+    parStore->temp.muteMask = midiWorker->muteMask.mid(0, midiWorker->maxNPoints);
     parStore->temp.loopMarker = getLoopMarker();
 
     parStore->tempToList(ix);
@@ -896,8 +898,10 @@ void SeqWidget::storeParams(int ix, bool empty)
 void SeqWidget::restoreParams(int ix)
 {
     if (parStore->list.at(ix).empty) return;
-    midiWorker->customWave = parStore->list.at(ix).wave;
-    midiWorker->muteMask = parStore->list.at(ix).muteMask;
+    for (int l1 = 0; l1 < parStore->list.at(ix).wave.count(); l1++) {
+        midiWorker->customWave.replace(l1, parStore->list.at(ix).wave.at(l1));
+        midiWorker->muteMask.replace(l1, parStore->list.at(ix).muteMask.at(l1));
+    }
     sizeBox->setCurrentIndex(parStore->list.at(ix).size);
     midiWorker->size = sizeBox->currentText().toInt();
     midiWorker->res = seqResValues[parStore->list.at(ix).res];
@@ -906,14 +910,13 @@ void SeqWidget::restoreParams(int ix)
     screen->setLoopMarker(parStore->list.at(ix).loopMarker);
 
     resBox->setCurrentIndex(parStore->list.at(ix).res);
-    waveFormBox->setCurrentIndex(parStore->list.at(ix).waveForm);
     loopBox->setCurrentIndex(parStore->list.at(ix).loopMode);
     notelength->setValue(parStore->list.at(ix).notelen);
     transpose->setValue(parStore->list.at(ix).transp);
     velocity->setValue(parStore->list.at(ix).vel);
     setDispVert(parStore->list.at(ix).dispVertical);
 
-    muteOut->setChecked(parStore->list.at(ix).muteOut);
+    //muteOut->setChecked(parStore->list.at(ix).muteOut);
     chIn->setCurrentIndex(parStore->list.at(ix).chIn);
     updateChIn(parStore->list.at(ix).chIn);
     channelOut->setCurrentIndex(parStore->list.at(ix).channelOut);
@@ -965,10 +968,9 @@ void SeqWidget::copyParamsFrom(SeqWidget *fromWidget)
     transpose->setValue(tmp);
 
     notelength->setValue(fromWidget->notelength->value());
-    midiWorker->customWave = fromWidget->getCustomWave();
-    midiWorker->muteMask.clear();
-    for (int l1 = 0; l1 < midiWorker->customWave.count(); l1++) {
-        midiWorker->muteMask.append(midiWorker->customWave.at(l1).muted);
+    for (int l1 = 0; l1 < fromWidget->getMidiWorker()->maxNPoints; l1++) {
+        midiWorker->customWave.replace(l1, fromWidget->getCustomWave().at(l1));
+        midiWorker->muteMask.replace(l1, midiWorker->customWave.at(l1).muted);
     }
     tmp = fromWidget->getLoopMarker();
     midiWorker->setLoopMarker(tmp);
@@ -997,65 +999,108 @@ void SeqWidget::handleController(int ccnumber, int channel, int value)
             switch (cclist.at(l2).ID) {
                 case 0: if (min == max) {
                             if (value == max) {
-                                m = muteOut->isChecked();
-                                muteOut->setChecked(!m);
+                                m = midiWorker->isMuted;
+                                midiWorker->setMuted(!m);
                             }
                         }
                         else {
                             if (value == max) {
-                                muteOut->setChecked(false);
+                                midiWorker->setMuted(false);
                             }
                             if (value == min) {
-                                muteOut->setChecked(true);
+                                midiWorker->setMuted(true);
                             }
                         }
                 break;
 
                 case 1:
                         sval = min + ((double)value * (max - min) / 127);
-                        velocity->setValue(sval);
+                        midiWorker->vel = sval;
                 break;
 
                 case 2:
                         sval = min + ((double)value * (max - min) / 127);
-                        notelength->setValue(sval);
+                        midiWorker->notelength = sval + sval;
                 break;
 
                 case 3: if (min == max) {
                             if (value == max) {
-                                m = recordAction->isChecked();
-                                recordAction->setChecked(!m);
+                                m = recordMode;
+                                setRecord(!m);
                                 return;
                             }
                         }
                         else {
                             if (value == max) {
-                                recordAction->setChecked(true);
+                                setRecord(true);
                             }
                             if (value == min) {
-                                recordAction->setChecked(false);
+                                setRecord(false);
                             }
                         }
                 break;
                 case 4:
                         sval = min + ((double)value * (max - min) / 127);
-                        resBox->setCurrentIndex(sval);
-                        updateRes(sval);
+                        if (sval < 5) resBoxIndex = sval;
                 break;
                 case 5:
                         sval = min + ((double)value * (max - min) / 127);
-                        sizeBox->setCurrentIndex(sval);
-                        updateSize(sval);
+                        if (sval < 8) sizeBoxIndex = sval;
                 break;
                 case 6:
                         sval = min + ((double)value * (max - min) / 127);
-                        loopBox->setCurrentIndex(sval);
-                        updateLoop(sval);
+                        if (sval < 6) midiWorker->curLoopMode = sval;
+                break;
+                case 7:
+                        sval = min + ((double)value * (max - min) / 127);
+                        if ((sval < parStore->list.count())
+                                && (sval != parStore->activeStore)
+                                && (sval != parStore->currentRequest)) {
+                            parStore->requestDispState(sval, 2);
+                            parStore->restoreRequest = sval;
+                            parStore->restoreRunOnce = (parStore->jumpToList.at(sval) > -2);
+                        }
+                        else return;
                 break;
 
                 default:
                 break;
             }
+            needsGUIUpdate = true;
         }
     }
 }
+
+void SeqWidget::updateDisplay()
+{
+    QVector<Sample> data;
+
+    parStore->updateDisplay(getCurrentIndex(), midiWorker->reverse);
+
+
+    if (dataChanged || midiWorker->dataChanged) {
+        dataChanged=false;
+        midiWorker->dataChanged=false;
+        midiWorker->getData(&data);
+        screen->updateData(data);
+        cursor->updateNumbers(midiWorker->res, midiWorker->size);
+    }
+    screen->updateDraw();
+    cursor->updateDraw();
+    midiControl->update();
+
+    if (!needsGUIUpdate) return;
+
+    transpose->setValue(midiWorker->transp);
+    notelength->setValue(midiWorker->notelength/2);
+    velocity->setValue(midiWorker->vel);
+    muteOut->setChecked(midiWorker->isMuted);
+    recordAction->setChecked(recordMode);
+    resBox->setCurrentIndex(resBoxIndex);
+    updateRes(resBoxIndex);
+    sizeBox->setCurrentIndex(sizeBoxIndex);
+    updateSize(sizeBoxIndex);
+    loopBox->setCurrentIndex(midiWorker->curLoopMode);
+
+    needsGUIUpdate = false;
+}
