@@ -32,16 +32,19 @@
 #include <QCheckBox>
 #include <QAction>
 #include <QToolButton>
+#include <QPushButton>
 #include <QVector>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
 
+#include "globstore.h"
 #include "midiseq.h"
 #include "slider.h"
 #include "seqscreen.h"
 #include "midicontrol.h"
 #include "managebox.h"
 #include "parstore.h"
+#include "cursor.h"
 
 
 /*!
@@ -60,25 +63,14 @@ class SeqWidget : public QWidget
 {
     Q_OBJECT
 
-    QAction *copyToCustomAction;
-
     MidiSeq *midiWorker;
+    GlobStore *globStore;
     QVector<Sample> data;
     bool modified;      /**< Is set to True if unsaved parameter modifications exist */
     bool lastMute;      /**< Contains the mute state of the last waveForm point modified by mouse click*/
-/*!
-* @brief This function populates the SeqWidget::waveForms list with
-* waveform names, currently only one.
-*
-* This might be used in the future for handling sequence presets
-*
-* @par Currently there is only one waveform
-*   - 0 Custom
-*
-*/
-    void loadWaveForms();
-
     bool recordMode;    /**< Is set to True if incoming notes are to be step-recorded*/
+    bool dataChanged;
+    bool needsGUIUpdate;
 
 /* PUBLIC MEMBERS */
   public:
@@ -92,21 +84,24 @@ class SeqWidget : public QWidget
  * @param mutedAdd If set to True, the module will be added in muted state
  * @param parent The parent widget of this module, i.e. MainWindow
  */
-    SeqWidget(MidiSeq *p_midiWorker, int portCount, bool compactStyle,
-            bool mutedAdd = false, QWidget* parent=0);
+    SeqWidget(MidiSeq *p_midiWorker, GlobStore *p_globStore,
+            int portCount, bool compactStyle,
+            bool mutedAdd = false, bool inOutVisible = true,
+            const QString& name = "", QWidget* parent=0);
     ~SeqWidget();
 
     ParStore *parStore;
     MidiControl *midiControl;
     SeqScreen *screen;
+    Cursor *cursor;
     ManageBox *manageBox;
 
-    QStringList waveForms;
     QComboBox *chIn;
     QComboBox *channelOut, *portOut;
-    QComboBox *waveFormBox, *resBox, *sizeBox, *freqBox;
+    QComboBox *resBox, *sizeBox, *freqBox;
     QComboBox *loopBox;
-    QCheckBox *muteOut;
+    QWidget *inOutBox;
+    QPushButton *muteOut;
     QCheckBox *enableNoteIn;
     QCheckBox *enableNoteOff;
     QCheckBox *enableVelIn;
@@ -116,7 +111,10 @@ class SeqWidget : public QWidget
     QCheckBox *dispVert[4];
     Slider *velocity, *transpose, *notelength;
     QAction *recordAction;
+    QAction* hideInOutBoxAction;
     int dispVertical;
+    int resBoxIndex;
+    int sizeBoxIndex;
 
     void setChIn(int value);
     void setEnableNoteIn(bool on);
@@ -145,20 +143,6 @@ class SeqWidget : public QWidget
 */
     void writeData(QXmlStreamWriter& xml);
 /*!
-* @brief This function stores some module parameters in a parameter
-* list object
-*
-* @param Position index in the parameter list
-*/
-    void storeParams(int ix, bool empty = false);
-/*!
-* @brief This function restores some module parameters from the parameter
-* list object
-*
-* @param Position index in the parameter list
-*/
-    void restoreParams(int ix);
-/*!
 * @brief This function copies all Seq module GUI parameters from
 * fromWidget
 *
@@ -261,19 +245,10 @@ class SeqWidget : public QWidget
     void setRecord(bool on);
     void setDispVert(int mode);
     void updateDispVert(int mode);
+    void setInOutBoxVisible(bool on);
+
+    void updateDisplay();
 
-/*!
-* @brief Slot currently not in use.
-*
-* This function calls the midi worker
-* MidiSeq::copyToCustom, which copies the
-* current sequence to the custom sequence buffer
-*
-* It switches the waveForm combobox to index 0 and calls
-* SeqWidget::updateWaveForm.
-*
-*/
-    void copyToCustom();
 /*!
 * @brief Receiver for incoming note events. This function either sets the
 * global transpose and velocity or records the received note.
@@ -350,6 +325,20 @@ class SeqWidget : public QWidget
 *
 */
     void setMuted(bool on);
+/*!
+* @brief This function stores some module parameters in a parameter
+* list object
+*
+* @param Position index in the parameter list
+*/
+    void storeParams(int ix, bool empty = false);
+/*!
+* @brief This function restores some module parameters from the parameter
+* list object
+*
+* @param Position index in the parameter list
+*/
+    void restoreParams(int ix);
 
     int getCurrentIndex() { return midiWorker->getCurrentIndex(); }
     int getLoopMarker() { return midiWorker->loopMarker; }
